# --------------------------------------------------------------------------------------------
# Copyright (c) Microsoft Corporation. All rights reserved.
# Licensed under the MIT License. See License.txt in the project root for license information.
#
# Code generated by aaz-dev-tools
# --------------------------------------------------------------------------------------------

# pylint: skip-file
# flake8: noqa

from azure.cli.core.aaz import *


@register_command(
    "netappfiles cache reset-smb-password",
)
class ResetSmbPassword(AAZCommand):
    """Resets the SMB password for the cache

    :example: Caches_ResetSmbPassword
        az netappfiles cache reset-smb-password --resource-group myResourceGroup --account-name account1 --pool-name pool1 --cache-name cache1
    """

    _aaz_info = {
        "version": "2026-04-01",
        "resources": [
            ["mgmt-plane", "/subscriptions/{}/resourcegroups/{}/providers/microsoft.netapp/netappaccounts/{}/capacitypools/{}/caches/{}/resetsmbpassword", "2026-04-01"],
        ]
    }

    AZ_SUPPORT_NO_WAIT = True

    def _handler(self, command_args):
        super()._handler(command_args)
        return self.build_lro_poller(self._execute_operations, self._output)

    _args_schema = None

    @classmethod
    def _build_arguments_schema(cls, *args, **kwargs):
        if cls._args_schema is not None:
            return cls._args_schema
        cls._args_schema = super()._build_arguments_schema(*args, **kwargs)

        # define Arg Group ""

        _args_schema = cls._args_schema
        _args_schema.account_name = AAZStrArg(
            options=["-a", "--account-name"],
            help="The name of the NetApp account",
            required=True,
            id_part="name",
            fmt=AAZStrArgFormat(
                pattern="^[a-zA-Z0-9][a-zA-Z0-9\\-_]{0,127}$",
            ),
        )
        _args_schema.cache_name = AAZStrArg(
            options=["-c", "--cache-name"],
            help="The name of the cache resource.",
            required=True,
            id_part="child_name_2",
            fmt=AAZStrArgFormat(
                pattern="^[a-zA-Z][a-zA-Z0-9\\-_]{0,63}$",
                max_length=64,
                min_length=3,
            ),
        )
        _args_schema.pool_name = AAZStrArg(
            options=["-p", "--pool-name"],
            help="The name of the capacity pool",
            required=True,
            id_part="child_name_1",
            fmt=AAZStrArgFormat(
                pattern="^[a-zA-Z0-9][a-zA-Z0-9\\-_]{0,63}$",
                max_length=64,
                min_length=1,
            ),
        )
        _args_schema.resource_group = AAZResourceGroupNameArg(
            required=True,
        )
        return cls._args_schema

    def _execute_operations(self):
        self.pre_operations()
        yield self.CachesResetSmbPassword(ctx=self.ctx)()
        self.post_operations()

    @register_callback
    def pre_operations(self):
        pass

    @register_callback
    def post_operations(self):
        pass

    def _output(self, *args, **kwargs):
        result = self.deserialize_output(self.ctx.vars.instance, client_flatten=True)
        return result

    class CachesResetSmbPassword(AAZHttpOperation):
        CLIENT_TYPE = "MgmtClient"

        def __call__(self, *args, **kwargs):
            request = self.make_request()
            session = self.client.send_request(request=request, stream=False, **kwargs)
            if session.http_response.status_code in [202]:
                return self.client.build_lro_polling(
                    self.ctx.args.no_wait,
                    session,
                    self.on_200,
                    self.on_error,
                    lro_options={"final-state-via": "location"},
                    path_format_arguments=self.url_parameters,
                )
            if session.http_response.status_code in [200]:
                return self.client.build_lro_polling(
                    self.ctx.args.no_wait,
                    session,
                    self.on_200,
                    self.on_error,
                    lro_options={"final-state-via": "location"},
                    path_format_arguments=self.url_parameters,
                )

            return self.on_error(session.http_response)

        @property
        def url(self):
            return self.client.format_url(
                "/subscriptions/{subscriptionId}/resourceGroups/{resourceGroupName}/providers/Microsoft.NetApp/netAppAccounts/{accountName}/capacityPools/{poolName}/caches/{cacheName}/resetSmbPassword",
                **self.url_parameters
            )

        @property
        def method(self):
            return "POST"

        @property
        def error_format(self):
            return "MgmtErrorFormat"

        @property
        def url_parameters(self):
            parameters = {
                **self.serialize_url_param(
                    "accountName", self.ctx.args.account_name,
                    required=True,
                ),
                **self.serialize_url_param(
                    "cacheName", self.ctx.args.cache_name,
                    required=True,
                ),
                **self.serialize_url_param(
                    "poolName", self.ctx.args.pool_name,
                    required=True,
                ),
                **self.serialize_url_param(
                    "resourceGroupName", self.ctx.args.resource_group,
                    required=True,
                ),
                **self.serialize_url_param(
                    "subscriptionId", self.ctx.subscription_id,
                    required=True,
                ),
            }
            return parameters

        @property
        def query_parameters(self):
            parameters = {
                **self.serialize_query_param(
                    "api-version", "2026-04-01",
                    required=True,
                ),
            }
            return parameters

        @property
        def header_parameters(self):
            parameters = {
                **self.serialize_header_param(
                    "Accept", "application/json",
                ),
            }
            return parameters

        def on_200(self, session):
            data = self.deserialize_http_content(session)
            self.ctx.set_var(
                "instance",
                data,
                schema_builder=self._build_schema_on_200
            )

        _schema_on_200 = None

        @classmethod
        def _build_schema_on_200(cls):
            if cls._schema_on_200 is not None:
                return cls._schema_on_200

            cls._schema_on_200 = AAZObjectType()

            _schema_on_200 = cls._schema_on_200
            _schema_on_200.etag = AAZStrType(
                flags={"read_only": True},
            )
            _schema_on_200.id = AAZStrType(
                flags={"read_only": True},
            )
            _schema_on_200.location = AAZStrType(
                flags={"required": True},
            )
            _schema_on_200.name = AAZStrType(
                flags={"read_only": True},
            )
            _schema_on_200.properties = AAZObjectType(
                flags={"required": True},
            )
            _schema_on_200.system_data = AAZObjectType(
                serialized_name="systemData",
                flags={"read_only": True},
            )
            _schema_on_200.tags = AAZDictType()
            _schema_on_200.type = AAZStrType(
                flags={"read_only": True},
            )
            _schema_on_200.zones = AAZListType()

            properties = cls._schema_on_200.properties
            properties.actual_throughput_mibps = AAZFloatType(
                serialized_name="actualThroughputMibps",
                flags={"read_only": True},
            )
            properties.cache_state = AAZStrType(
                serialized_name="cacheState",
                flags={"read_only": True},
            )
            properties.cache_subnet_resource_id = AAZStrType(
                serialized_name="cacheSubnetResourceId",
                flags={"required": True},
            )
            properties.cifs_change_notifications = AAZStrType(
                serialized_name="cifsChangeNotifications",
            )
            properties.encryption = AAZStrType(
                flags={"read_only": True},
            )
            properties.encryption_key_source = AAZStrType(
                serialized_name="encryptionKeySource",
                flags={"required": True},
            )
            properties.export_policy = AAZObjectType(
                serialized_name="exportPolicy",
            )
            properties.file_access_logs = AAZStrType(
                serialized_name="fileAccessLogs",
                flags={"read_only": True},
            )
            properties.file_path = AAZStrType(
                serialized_name="filePath",
                flags={"required": True},
            )
            properties.global_file_locking = AAZStrType(
                serialized_name="globalFileLocking",
            )
            properties.kerberos = AAZStrType()
            properties.key_vault_private_endpoint_resource_id = AAZStrType(
                serialized_name="keyVaultPrivateEndpointResourceId",
            )
            properties.language = AAZStrType(
                flags={"read_only": True},
            )
            properties.ldap = AAZStrType()
            properties.ldap_server_type = AAZStrType(
                serialized_name="ldapServerType",
            )
            properties.maximum_number_of_files = AAZIntType(
                serialized_name="maximumNumberOfFiles",
                flags={"read_only": True},
            )
            properties.mount_targets = AAZListType(
                serialized_name="mountTargets",
                flags={"read_only": True},
            )
            properties.origin_cluster_information = AAZObjectType(
                serialized_name="originClusterInformation",
                flags={"required": True},
            )
            properties.peering_subnet_resource_id = AAZStrType(
                serialized_name="peeringSubnetResourceId",
                flags={"required": True},
            )
            properties.protocol_types = AAZListType(
                serialized_name="protocolTypes",
            )
            properties.provisioning_state = AAZStrType(
                serialized_name="provisioningState",
                flags={"read_only": True},
            )
            properties.size = AAZIntType(
                flags={"required": True},
            )
            properties.smb_settings = AAZObjectType(
                serialized_name="smbSettings",
            )
            properties.throughput_mibps = AAZFloatType(
                serialized_name="throughputMibps",
            )
            properties.write_back = AAZStrType(
                serialized_name="writeBack",
            )

            export_policy = cls._schema_on_200.properties.export_policy
            export_policy.rules = AAZListType()

            rules = cls._schema_on_200.properties.export_policy.rules
            rules.Element = AAZObjectType()

            _element = cls._schema_on_200.properties.export_policy.rules.Element
            _element.allowed_clients = AAZStrType(
                serialized_name="allowedClients",
            )
            _element.chown_mode = AAZStrType(
                serialized_name="chownMode",
            )
            _element.cifs = AAZBoolType()
            _element.has_root_access = AAZBoolType(
                serialized_name="hasRootAccess",
            )
            _element.kerberos5_read_only = AAZBoolType(
                serialized_name="kerberos5ReadOnly",
            )
            _element.kerberos5_read_write = AAZBoolType(
                serialized_name="kerberos5ReadWrite",
            )
            _element.kerberos5i_read_only = AAZBoolType(
                serialized_name="kerberos5iReadOnly",
            )
            _element.kerberos5i_read_write = AAZBoolType(
                serialized_name="kerberos5iReadWrite",
            )
            _element.kerberos5p_read_only = AAZBoolType(
                serialized_name="kerberos5pReadOnly",
            )
            _element.kerberos5p_read_write = AAZBoolType(
                serialized_name="kerberos5pReadWrite",
            )
            _element.nfsv3 = AAZBoolType()
            _element.nfsv41 = AAZBoolType()
            _element.rule_index = AAZIntType(
                serialized_name="ruleIndex",
            )
            _element.unix_read_only = AAZBoolType(
                serialized_name="unixReadOnly",
            )
            _element.unix_read_write = AAZBoolType(
                serialized_name="unixReadWrite",
            )

            mount_targets = cls._schema_on_200.properties.mount_targets
            mount_targets.Element = AAZObjectType()

            _element = cls._schema_on_200.properties.mount_targets.Element
            _element.ip_address = AAZStrType(
                serialized_name="ipAddress",
                flags={"read_only": True},
            )
            _element.mount_target_id = AAZStrType(
                serialized_name="mountTargetId",
                flags={"read_only": True},
            )
            _element.smb_server_fqdn = AAZStrType(
                serialized_name="smbServerFqdn",
                flags={"read_only": True},
            )

            origin_cluster_information = cls._schema_on_200.properties.origin_cluster_information
            origin_cluster_information.peer_addresses = AAZListType(
                serialized_name="peerAddresses",
                flags={"required": True},
            )
            origin_cluster_information.peer_cluster_name = AAZStrType(
                serialized_name="peerClusterName",
                flags={"required": True},
            )
            origin_cluster_information.peer_volume_name = AAZStrType(
                serialized_name="peerVolumeName",
                flags={"required": True},
            )
            origin_cluster_information.peer_vserver_name = AAZStrType(
                serialized_name="peerVserverName",
                flags={"required": True},
            )

            peer_addresses = cls._schema_on_200.properties.origin_cluster_information.peer_addresses
            peer_addresses.Element = AAZStrType()

            protocol_types = cls._schema_on_200.properties.protocol_types
            protocol_types.Element = AAZStrType()

            smb_settings = cls._schema_on_200.properties.smb_settings
            smb_settings.smb_access_based_enumeration = AAZStrType(
                serialized_name="smbAccessBasedEnumeration",
            )
            smb_settings.smb_encryption = AAZStrType(
                serialized_name="smbEncryption",
            )
            smb_settings.smb_non_browsable = AAZStrType(
                serialized_name="smbNonBrowsable",
            )

            system_data = cls._schema_on_200.system_data
            system_data.created_at = AAZStrType(
                serialized_name="createdAt",
            )
            system_data.created_by = AAZStrType(
                serialized_name="createdBy",
            )
            system_data.created_by_type = AAZStrType(
                serialized_name="createdByType",
            )
            system_data.last_modified_at = AAZStrType(
                serialized_name="lastModifiedAt",
            )
            system_data.last_modified_by = AAZStrType(
                serialized_name="lastModifiedBy",
            )
            system_data.last_modified_by_type = AAZStrType(
                serialized_name="lastModifiedByType",
            )

            tags = cls._schema_on_200.tags
            tags.Element = AAZStrType()

            zones = cls._schema_on_200.zones
            zones.Element = AAZStrType()

            return cls._schema_on_200


class _ResetSmbPasswordHelper:
    """Helper class for ResetSmbPassword"""


__all__ = ["ResetSmbPassword"]



# --------------------------------------------------------------------------------------------
# Copyright (c) Microsoft Corporation. All rights reserved.
# Licensed under the MIT License. See License.txt in the project root for license information.
# --------------------------------------------------------------------------------------------

from azure.cli.core.commands import CliCommandType
from azure.cli.command_modules.maps._client_factory import cf_accounts, cf_map


def load_command_table(self, _):
    mgmt_type = CliCommandType(
        operations_tmpl='azure.mgmt.maps.operations._accounts_operations#AccountsOperations.{}',
        client_factory=cf_accounts)

    with self.command_group('maps account', mgmt_type) as g:
        g.show_command('show', 'get')
        g.custom_command('list', 'list_accounts')
        g.custom_command('create', 'maps_account_create')
        g.command('delete', 'delete')
        g.custom_command('update', 'maps_account_update')

    with self.command_group('maps account keys', mgmt_type) as g:
        g.custom_command('renew', 'maps_account_regenerate_key')
        g.custom_command('list', 'maps_account_list_key')

    maps_map = CliCommandType(
        operations_tmpl='azure.mgmt.maps.operations._maps_operations#MapsOperations.{}',
        client_factory=cf_map)

    with self.command_group('maps map', maps_map, client_factory=cf_map) as g:
        g.custom_command('list-operation', 'maps_map_list_operation')



# --------------------------------------------------------------------------------------------
# Copyright (c) Microsoft Corporation. All rights reserved.
# Licensed under the MIT License. See License.txt in the project root for license information.
# --------------------------------------------------------------------------------------------
# pylint: disable=unused-import

""" AAZ is short for Atomic Azure CLI, which is an atomic command layer of azure cli.
The command in atomic layer is called atomic command, mapping to an operation of azure resource.
Atomic commands can be generated from rest api by using aaz-dev tool.
"""

from ._arg import AAZArgumentsSchema, AAZArgEnum, AAZStrArg, AAZIntArg, AAZObjectArg, AAZDictArg, \
    AAZFreeFormDictArg, AAZFloatArg, AAZBaseArg, AAZBoolArg, AAZListArg, AAZResourceGroupNameArg, \
    AAZResourceLocationArg, AAZResourceIdArg, AAZSubscriptionIdArg, AAZUuidArg, AAZDateArg, AAZTimeArg, \
    AAZDateTimeArg, AAZDurationArg, AAZFileArg, AAZPasswordArg, AAZPaginationTokenArg, AAZPaginationLimitArg, \
    AAZAnyTypeArg
from ._arg_fmt import AAZStrArgFormat, AAZIntArgFormat, AAZFloatArgFormat, AAZBoolArgFormat, AAZObjectArgFormat, \
    AAZDictArgFormat, AAZFreeFormDictArgFormat, AAZListArgFormat, AAZResourceLocationArgFormat, \
    AAZResourceIdArgFormat, AAZSubscriptionIdArgFormat, AAZUuidFormat, AAZDateFormat, AAZTimeFormat, \
    AAZDateTimeFormat, AAZDurationFormat, AAZFileArgTextFormat, AAZFileArgBase64EncodeFormat, AAZFileArgFormat, \
    AAZPaginationTokenArgFormat
from ._base import has_value, AAZValuePatch, AAZUndefined
from ._command import AAZCommand, AAZWaitCommand, AAZCommandGroup, \
    register_callback, register_command, register_command_group, load_aaz_command_table, \
    load_aaz_command_table_args_guided, link_helper
from ._field_type import AAZIntType, AAZFloatType, AAZStrType, AAZBoolType, AAZDictType, AAZFreeFormDictType, \
    AAZListType, AAZObjectType, AAZIdentityObjectType, AAZAnyType
from ._operation import AAZHttpOperation, AAZJsonInstanceUpdateOperation, AAZGenericInstanceUpdateOperation, \
    AAZJsonInstanceDeleteOperation, AAZJsonInstanceCreateOperation
from ._prompt import AAZPromptInput, AAZPromptPasswordInput
from ._selector import AAZJsonSelector
from ._client import AAZBaseClient, AAZClientConfiguration, register_client, CloudNameEnum



# --------------------------------------------------------------------------------------------
# Copyright (c) Microsoft Corporation. All rights reserved.
# Licensed under the MIT License. See License.txt in the project root for license information.
# --------------------------------------------------------------------------------------------
import math
import re

from dateutil import parser
from functools import cmp_to_key
from requests import get
from knack.log import get_logger
from knack.prompting import NoTTYException, prompt_pass
from knack.util import CLIError
from azure.cli.core.azclierror import ArgumentUsageError, RequiredArgumentMissingError, ValidationError
from azure.cli.core.commands.client_factory import get_mgmt_service_client, get_subscription_id
from azure.cli.core.commands.validators import (
    get_default_location_from_resource_group, validate_tags)
from azure.cli.core.profiles import ResourceType
from azure.cli.core.util import parse_proxy_resource_id
from azure.core.exceptions import HttpResponseError
from azure.mgmt import postgresqlflexibleservers as postgresql_flexibleservers
from azure.mgmt.core.tools import (
    is_valid_resource_id,
    is_valid_resource_name,
    parse_resource_id,
    resource_id)
from .._client_factory import cf_postgres_check_resource_availability, cf_postgres_flexible_servers
from .._config_reader import get_cloud_cluster
from ._flexible_server_location_capabilities_util import (
    get_performance_tiers,
    get_performance_tiers_for_storage,
    get_postgres_location_capability_info,
    get_postgres_server_capability_info)
from ._flexible_server_util import (
    _is_resource_name,
    get_id_components,
    get_postgres_skus,
    get_postgres_storage_sizes,
    get_postgres_tiers,
    get_user_confirmation,
    parse_public_access_input)

logger = get_logger(__name__)
IP_ADDRESS_CHECKER = 'https://api.ipify.org'


# pylint: disable=import-outside-toplevel, raise-missing-from, unbalanced-tuple-unpacking
def _get_resource_group_from_server_name(cli_ctx, server_name):
    """
    Fetch resource group from server name
    :param str server_name: name of the server
    :return: resource group name or None
    :rtype: str
    """

    client = get_mgmt_service_client(cli_ctx, ResourceType.MGMT_POSTGRESQL).servers
    for server in client.list():
        id_comps = parse_resource_id(server.id)
        if id_comps['name'] == server_name:
            return id_comps['resource_group']
    return None


def get_combined_validator(validators):
    def _final_validator_impl(cmd, namespace):
        # do additional creation validation
        verbs = cmd.name.rsplit(' ', 2)
        if verbs[1] == 'server' and verbs[2] == 'create':
            password_validator(namespace)
            get_default_location_from_resource_group(cmd, namespace)

        validate_tags(namespace)

        for validator in validators:
            validator(namespace)

    return _final_validator_impl


def configuration_value_validator(ns):
    val = ns.value
    if val is None or not val.strip():
        ns.value = None
        ns.source = 'system-default'


def tls_validator(ns):
    if ns.minimal_tls_version:
        if ns.ssl_enforcement is not None and ns.ssl_enforcement.lower() != 'enabled':
            raise CLIError('Cannot specify --minimal-tls-version when --ssl-enforcement is set to "Disabled".')


def password_validator(ns):
    if not ns.administrator_login_password:
        try:
            ns.administrator_login_password = prompt_pass(msg='Admin Password: ')
        except NoTTYException:
            raise CLIError('Specify --administrator-login-password when running in non-interactive mode.')


def retention_validator(ns):
    if ns.backup_retention is not None:
        val = ns.backup_retention
        if not 7 <= int(val) <= 35:
            raise CLIError('Invalid value for --backup-retention. Allowed values: 7 to 35 days.')


def db_renaming_cluster_validator(ns):
    if ns.database_name is not None and ns.cluster_size is None:
        raise ArgumentUsageError('The --database-name argument can only be used '
                                 'when --node-count is present, as it only applies to elastic clusters.')


# Validates if a subnet id or name have been given by the user. If subnet id is given, vnet-name should not be provided.
def validate_subnet(cmd, namespace):

    subnet = namespace.virtual_network_subnet_id
    subnet_is_id = is_valid_resource_id(subnet)
    vnet = namespace.vnet_name

    if (subnet_is_id and not vnet) or (not subnet and not vnet):
        pass
    elif subnet and not subnet_is_id and vnet:
        namespace.virtual_network_subnet_id = resource_id(
            subscription=get_subscription_id(cmd.cli_ctx),
            resource_group=namespace.resource_group_name,
            namespace='Microsoft.Network',
            type='virtualNetworks',
            name=vnet,
            child_type_1='subnets',
            child_name_1=subnet)
    else:
        raise CLIError('Specify either --subnet <subnet-identifier> or --subnet <subnet-name> --vnet-name <vnet-name>.')
    delattr(namespace, 'vnet_name')


def validate_private_endpoint_connection_id(cmd, namespace):
    if namespace.connection_id:
        result = parse_proxy_resource_id(namespace.connection_id)
        namespace.private_endpoint_connection_name = result['child_name_1']
        namespace.server_name = result['name']
        namespace.resource_group_name = result['resource_group']
    if namespace.server_name and not namespace.resource_group_name:
        namespace.resource_group_name = _get_resource_group_from_server_name(cmd.cli_ctx, namespace.server_name)

    if not all([namespace.server_name, namespace.resource_group_name, namespace.private_endpoint_connection_name]):
        raise CLIError('Specify either --id <private-endpoint-connection-identifier> '
                       'or --name <private-endpoint-connection-name> --server-name <server-name>.')

    del namespace.connection_id


# pylint: disable=too-many-locals
def pg_arguments_validator(db_context, location, tier, sku_name, storage_gb, server_name=None, database_name=None,
                           zone=None, standby_availability_zone=None,
                           zonal_resiliency=None, allow_same_zone=False, subnet=None,
                           public_access=None, version=None, instance=None, geo_redundant_backup=None,
                           byok_identity=None, byok_key=None, backup_byok_identity=None, backup_byok_key=None,
                           federated_client_id=None, backup_federated_client_id=None,
                           auto_grow=None, performance_tier=None,
                           storage_type=None, iops=None, throughput=None, cluster_size=None,
                           password_auth=None, microsoft_entra_auth=None,
                           admin_name=None, admin_id=None, admin_type=None):
    validate_server_name(db_context, server_name, 'Microsoft.DBforPostgreSQL/flexibleServers')
    validate_database_name(database_name)
    is_create = not instance
    if is_create:
        list_location_capability_info = get_postgres_location_capability_info(
            db_context.cmd,
            location)
    else:
        list_location_capability_info = get_postgres_server_capability_info(
            db_context.cmd,
            resource_group=parse_resource_id(instance.id)["resource_group"],
            server_name=instance.name)
    sku_info = list_location_capability_info['sku_info']
    sku_info = {k.lower(): v for k, v in sku_info.items()}
    single_az = list_location_capability_info['single_az']
    geo_backup_supported = list_location_capability_info['geo_backup_supported']
    _cluster_validator(cluster_size, instance)
    _network_arg_validator(subnet, public_access)
    _pg_tier_validator(tier, sku_info)  # need to be validated first
    if tier is None and instance is not None:
        tier = instance.sku.tier.lower()
    if "supported_storageV2_size" in sku_info[tier.lower()]:
        supported_storageV2_size = sku_info[tier.lower()]["supported_storageV2_size"]
    else:
        supported_storageV2_size = None
    _pg_storage_type_validator(storage_type, auto_grow, performance_tier,
                               tier, supported_storageV2_size, iops, throughput, instance, version)
    _pg_storage_performance_tier_validator(performance_tier,
                                           sku_info,
                                           tier,
                                           instance.storage.storage_size_gb if storage_gb is None else storage_gb)
    if geo_redundant_backup is None and instance is not None:
        geo_redundant_backup = instance.backup.geo_redundant_backup
    _pg_georedundant_backup_validator(geo_redundant_backup, geo_backup_supported)
    _pg_storage_validator(storage_gb, sku_info, tier, storage_type, iops, throughput, instance)
    _pg_sku_name_validator(sku_name, sku_info, tier, instance)
    _pg_zonal_resiliency_validator(zonal_resiliency, allow_same_zone,
                                   standby_availability_zone, zone, tier, single_az, instance)
    pg_version_validator(version, list_location_capability_info['server_versions'])
    pg_byok_validator(byok_identity, byok_key, backup_byok_identity, backup_byok_key,
                      geo_redundant_backup, instance,
                      federated_client_id=federated_client_id,
                      backup_federated_client_id=backup_federated_client_id)
    is_microsoft_entra_auth = bool(microsoft_entra_auth is not None and microsoft_entra_auth.lower() == 'enabled')
    _pg_authentication_validator(password_auth, is_microsoft_entra_auth,
                                 admin_name, admin_id, admin_type, instance)


def _cluster_validator(cluster_size, instance):
    if cluster_size is not None and instance and not instance.cluster:
        raise ValidationError('Node count can only be specified for an elastic cluster.')


def _pg_storage_validator(storage_gb, sku_info, tier, storage_type, iops, throughput, instance):
    is_ssdv2 = ((storage_type and storage_type.lower() == 'premiumv2_lrs') or
                (instance and instance.storage.type and
                 instance.storage.type.lower() == 'premiumv2_lrs'))
    # storage_gb range validation
    if storage_gb is not None:
        if instance is not None:
            original_size = instance.storage.storage_size_gb
            if original_size > storage_gb:
                raise CLIError('Decreasing storage size is not supported. Current storage size is {} GiB, '
                               'and you are trying to set it to {} GiB.'.format(original_size, storage_gb))
        if not is_ssdv2:
            storage_sizes = get_postgres_storage_sizes(sku_info, tier)
            if storage_gb not in storage_sizes:
                storage_sizes = sorted([int(size) for size in storage_sizes])
                raise CLIError('Invalid value for --storage-size. Allowed values (in GiB): {}.'
                               .format(storage_sizes))

    # ssdv2 range validation
    if is_ssdv2 and (storage_gb is not None or throughput is not None or iops is not None):
        _valid_ssdv2_range(storage_gb, sku_info, tier, iops, throughput, instance)


def _valid_ssdv2_range(storage_gb, sku_info, tier, iops, throughput, instance):
    storage_gib = storage_gb if storage_gb is not None else instance.storage.storage_size_gb
    storage_iops = iops if iops is not None else instance.storage.iops
    storage_throughput = throughput if throughput is not None else instance.storage.throughput

    # find min and max values for storage
    sku_tier = tier.lower()
    supported_storageV2_size = sku_info[sku_tier]["supported_storageV2_size"]
    min_storage = instance.storage.storage_size_gb if instance is not None else supported_storageV2_size
    max_storage = sku_info[sku_tier]["supported_storageV2_size_max"]
    if not min_storage <= storage_gib <= max_storage:
        raise CLIError('The requested value for storage size does not fall between {} and {} GiB.'
                       .format(min_storage, max_storage))

    storage = storage_gib * 1.07374182
    # find min and max values for IOPS
    min_iops = sku_info[sku_tier]["supported_storageV2_iops"]
    supported_max_iops = sku_info[sku_tier]["supported_storageV2_iops_max"]
    calculated_max_iops = math.floor(max(0, storage - 6) * 500 + min_iops)
    max_iops = min(supported_max_iops, calculated_max_iops)

    if not min_iops <= storage_iops <= max_iops:
        raise CLIError('The requested value for IOPS does not fall between {} and {} operations/sec.'
                       .format(min_iops, max_iops))

    # find min and max values for throughput
    min_throughput = sku_info[sku_tier]["supported_storageV2_throughput"]
    supported_max_throughput = sku_info[sku_tier]["supported_storageV2_throughput_max"]
    if storage > 6:
        max_storage_throughput = math.floor(max(0.25 * storage_iops, min_throughput))
    else:
        max_storage_throughput = min_throughput
    max_throughput = min(supported_max_throughput, max_storage_throughput)

    if not min_throughput <= storage_throughput <= max_throughput:
        raise CLIError('The requested value for throughput does not fall between {} and {} MB/sec.'
                       .format(min_throughput, max_throughput))


def _pg_tier_validator(tier, sku_info):
    if tier:
        tiers = [item.lower() for item in get_postgres_tiers(sku_info)]
        if tier.lower() not in tiers:
            raise CLIError('Invalid value for --tier. Allowed values: {}'.format(tiers))


def compare_sku_names(sku_1, sku_2):
    regex_pattern = r"\D+(?P<core_number>\d+)\D+(?P<version>\d*)"

    sku_1_match = re.search(regex_pattern, sku_1)
    sku_2_match = re.search(regex_pattern, sku_2)

    # the case where version number is different, sort by the version number first
    if sku_1_match.group('version') and int(sku_2_match.group('version')) > int(sku_1_match.group('version')):
        return 1
    if sku_1_match.group('version') and int(sku_2_match.group('version')) < int(sku_1_match.group('version')):
        return -1

    # the case where version number is the same, we want to sort by the core number
    if int(sku_2_match.group('core_number')) < int(sku_1_match.group('core_number')):
        return 1
    if int(sku_2_match.group('core_number')) > int(sku_1_match.group('core_number')):
        return -1

    return 0


def _pg_sku_name_validator(sku_name, sku_info, tier, instance):
    additional_error = ''
    if instance is not None:
        tier = instance.sku.tier if tier is None else tier
    else:
        additional_error = 'When --tier is not specified, it defaults to GeneralPurpose. '
    if sku_name:
        skus = [item.lower() for item in get_postgres_skus(sku_info, tier.lower())]
        if sku_name.lower() not in skus:
            raise CLIError('Invalid value for --sku-name. The SKU name is not available in the {} tier. {}'
                           'Provide a valid SKU name for this tier, or specify --tier with the correct tier. '
                           'Allowed values: {}'
                           .format(tier, additional_error, sorted(skus, key=cmp_to_key(compare_sku_names))))


def _pg_storage_performance_tier_validator(performance_tier, sku_info, tier=None, storage_size=None):
    if performance_tier:
        tiers = get_postgres_tiers(sku_info)
        if tier.lower() in [item.lower() for item in tiers]:
            if storage_size is None:
                performance_tiers = [item.lower() for item in
                                     get_performance_tiers(sku_info[tier.lower()]["storage_edition"])]
            else:
                performance_tiers = [item.lower() for item in
                                     get_performance_tiers_for_storage(sku_info[tier.lower()]["storage_edition"],
                                     storage_size=storage_size)]

            if performance_tier.lower() not in performance_tiers:
                raise CLIError('Invalid value for --performance-tier for storage size {}. '
                               'Allowed values: {}'.format(storage_size, performance_tiers))


def pg_version_validator(version, versions):
    if version:
        if version not in versions:
            raise CLIError('Invalid value for --version. Allowed values: {}'.format(sorted(versions)))
        if version in ('11', '12', '13'):
            logger.warning('The version selected is a retired community version of PostgreSQL. '
                           'To use this version, you will automatically be enrolled in our extended '
                           'support plan for an additional charge starting August 1, 2026. '
                           'Upgrade to PostgreSQL 14 or later as soon as possible to '
                           'maintain security, performance, and supportability.')


def _pg_zonal_resiliency_validator(zonal_resiliency, allow_same_zone,
                                   standby_availability_zone, zone, tier, single_az, instance):
    zonal_resiliency_enabled = (zonal_resiliency is not None and zonal_resiliency.lower() != 'disabled')

    if instance:
        tier = instance.sku.tier if tier is None else tier
        zone = instance.availability_zone if zone is None else zone

    if zonal_resiliency_enabled:
        if tier.lower() == 'burstable':
            raise ArgumentUsageError('High availability is not supported for the Burstable tier.')
        if single_az and allow_same_zone is False:
            raise ArgumentUsageError('This location has a single availability zone. '
                                     'To proceed, set --allow-same-zone.')

    if standby_availability_zone:
        if not zonal_resiliency_enabled:
            raise ArgumentUsageError('To set --standby-zone, enable --zonal-resiliency.')
        if zone == standby_availability_zone:
            raise ArgumentUsageError('Your server is in availability zone {}. '
                                     'The standby availability zone must be different from the server zone.'
                                     .format(zone))

    if allow_same_zone and not zonal_resiliency_enabled:
        raise ArgumentUsageError('You can only set --allow-same-zone when --zonal-resiliency is set to "Enabled".')


def _pg_georedundant_backup_validator(geo_redundant_backup, geo_backup_supported):
    if (geo_redundant_backup and geo_redundant_backup.lower() == 'enabled') and not geo_backup_supported:
        raise ArgumentUsageError('This location does not support geo-redundant backup.')


def pg_byok_validator(byok_identity, byok_key, backup_byok_identity=None, backup_byok_key=None,
                      geo_redundant_backup=None, instance=None,
                      federated_client_id=None, backup_federated_client_id=None):

    if bool(byok_identity is None) ^ bool(byok_key is None):
        raise ArgumentUsageError('A user-assigned identity and Key Vault key must be provided together. '
                                 'Provide --identity and --key together.')

    if bool(backup_byok_identity is None) ^ bool(backup_byok_key is None):
        raise ArgumentUsageError('A user-assigned identity and Key Vault key must be provided together. '
                                 'Provide --backup-identity and --backup-key together.')

    if bool(byok_identity is not None) and bool(backup_byok_identity is not None) and \
       byok_identity.lower() == backup_byok_identity.lower():
        raise ArgumentUsageError('The primary user-assigned identity and backup identity cannot be the same. '
                                 'Provide different identities for --identity and --backup-identity.')

    if (federated_client_id or backup_federated_client_id) and byok_identity is None:
        if instance is None:
            raise ArgumentUsageError('To use --federated-client-id or --geo-backup-federated-client-id, '
                                     'provide --identity and --key together.')
        if not (instance.data_encryption and instance.data_encryption.type == 'AzureKeyVault'):
            logger.warning('You cannot update data encryption properties on a server '
                           'that was not created with data encryption..')

    if bool(federated_client_id is not None) and bool(backup_federated_client_id is not None) and \
       federated_client_id.lower() == backup_federated_client_id.lower():
        raise ArgumentUsageError('The primary federated client ID and backup federated client ID cannot be the same. '
                                 'Provide different IDs for --federated-client-id and '
                                 '--geo-backup-federated-client-id.')

    if (instance is not None) and \
       not (instance.data_encryption and instance.data_encryption.type == 'AzureKeyVault') and \
       (byok_key or backup_byok_key):
        raise ArgumentUsageError('You cannot enable data encryption on a server '
                                 'that was not created with data encryption.')

    if geo_redundant_backup is None or geo_redundant_backup.lower() == 'disabled':
        if backup_byok_identity or backup_byok_key:
            raise ArgumentUsageError('Geo-redundant backup is not enabled. '
                                     'Do not provide --backup-identity or --backup-key.')
    else:
        if instance is None and (bool(byok_key is not None) ^ bool(backup_byok_key is not None)):
            raise ArgumentUsageError('To enable data encryption for geo-redundant backup, provide both the '
                                     'primary and geo-backup user-assigned identities and Key Vault keys.')
        if instance is not None and (bool(byok_identity is None) ^ bool(backup_byok_identity is None)):
            primary_user_assigned_identity_id = byok_identity if byok_identity else \
                instance.data_encryption.primary_user_assigned_identity_id
            geo_backup_user_assigned_identity_id = backup_byok_identity if backup_byok_identity else \
                instance.data_encryption.geo_backup_user_assigned_identity_id
            if primary_user_assigned_identity_id.lower() == geo_backup_user_assigned_identity_id.lower():
                raise ArgumentUsageError('The primary user-assigned identity and backup identity cannot be the same. '
                                         'Provide different identities for --identity and --backup-identity.')


def _network_arg_validator(subnet, public_access):
    if subnet is not None and public_access is not None:
        raise CLIError('The --subnet and --public-access arguments cannot be used together. '
                       'Use only one of them.')


def maintenance_window_validator(ns):
    options = ["sun", "mon", "tue", "wed", "thu", "fri", "sat", "disabled"]
    if ns.maintenance_window:
        parsed_input = ns.maintenance_window.split(':')
        if not parsed_input or len(parsed_input) > 3:
            raise CLIError('Invalid value for --maintenance-window. '
                           'Use <Day>:<Hour>:<Minute>. Example: "Mon:8:30" schedules maintenance '
                           'on Monday at 8:30 UTC.')
        if len(parsed_input) >= 1 and parsed_input[0].lower() not in options:
            raise CLIError('Invalid value for --maintenance-window. '
                           'The first value must be a day of the week or "Disabled". '
                           'Allowed values: {"Sun", "Mon", "Tue", "Wed", "Thu", "Fri", "Sat"}.')
        if len(parsed_input) >= 2 and \
           (not parsed_input[1].isdigit() or int(parsed_input[1]) < 0 or int(parsed_input[1]) > 23):
            raise CLIError('Invalid value for --maintenance-window. '
                           'The hour must be between 0 and 23. Allowed values: {0, 1, ... 23}.')
        if len(parsed_input) >= 3 and \
           (not parsed_input[2].isdigit() or int(parsed_input[2]) < 0 or int(parsed_input[2]) > 59):
            raise CLIError('Invalid value for --maintenance-window. '
                           'The minute must be between 0 and 59. Allowed values: {0, 1, ... 59}.')


def ip_address_validator(ns):
    if (ns.end_ip_address and not _validate_ip(ns.end_ip_address)) or \
       (ns.start_ip_address and not _validate_ip(ns.start_ip_address)):
        raise CLIError('Invalid IP address. Provide an IPv4 address, for example 12.12.12.12.')
    if ns.start_ip_address and ns.end_ip_address:
        _validate_start_and_end_ip_address_order(ns.start_ip_address, ns.end_ip_address)


def public_access_validator(ns):
    if ns.public_access:
        val = ns.public_access.lower()
        if not (val in ['disabled', 'enabled', 'all', 'none'] or
                (len(val.split('-')) == 1 and _validate_ip(val)) or
                (len(val.split('-')) == 2 and _validate_ip(val))):
            raise CLIError('Invalid value for --public-access. '
                           'Allowed values: \'Disabled\', \'Enabled\', \'All\', \'None\', \'<startIP>\', '
                           'or \'<startIP>-<endIP>\', where each IP ranges from 0.0.0.0 to 255.255.255.255.')
        if len(val.split('-')) == 2:
            vals = val.split('-')
            _validate_start_and_end_ip_address_order(vals[0], vals[1])


def _validate_start_and_end_ip_address_order(start_ip, end_ip):
    start_ip_elements = [int(octet) for octet in start_ip.split('.')]
    end_ip_elements = [int(octet) for octet in end_ip.split('.')]

    for idx in range(4):
        if start_ip_elements[idx] < end_ip_elements[idx]:
            break
        if start_ip_elements[idx] > end_ip_elements[idx]:
            raise ArgumentUsageError('The end IP address is smaller than the start IP address.')


def _validate_ip(ips):
    """
    # Regex not working for re.(regex, '255.255.255.255'). Hence commenting it out for now
    regex = r'^(25[0-5]|2[0-4][0-9]|[0-1]?[0-9][0-9]?).(
                25[0-5]|2[0-4][0-9]|[0-1]?[0-9][0-9]?).(
                25[0-5]|2[0-4][0-9]|[0-1]?[0-9][0-9]?).(
                25[0-5]|2[0-4][0-9]|[0-1]?[0-9][0-9]?)$'
    """
    parsed_input = ips.split('-')
    if len(parsed_input) == 1:
        return _validate_ranges_in_ip(parsed_input[0])
    if len(parsed_input) == 2:
        return _validate_ranges_in_ip(parsed_input[0]) and _validate_ranges_in_ip(parsed_input[1])
    return False


def _validate_ranges_in_ip(ip):
    parsed_ip = ip.split('.')
    if len(parsed_ip) == 4 and _valid_range(parsed_ip[0]) and _valid_range(parsed_ip[1]) \
       and _valid_range(parsed_ip[2]) and _valid_range(parsed_ip[3]):
        return True
    return False


def _valid_range(addr_range):
    if addr_range.isdigit() and 0 <= int(addr_range) <= 255:
        return True
    return False


def virtual_endpoint_name_validator(ns):
    if not re.search(r'^(?=[a-z0-9].*)(?=.*[a-z-])(?!.*[^a-z0-9-])(?=.*[a-z0-9]$)', ns.virtual_endpoint_name):
        raise ValidationError('The virtual endpoint name can only contain 0-9, a-z, and \'-\'. '
                              'The virtual endpoint name must not start or end in a hyphen. '
                              'Additionally, the name of the virtual endpoint must be at least 3 characters '
                              'and no more than 63 characters in length. ')


def firewall_rule_name_validator(ns):
    if not ns.firewall_rule_name:
        return
    if not re.search(r'^[a-zA-Z0-9][-_a-zA-Z0-9]{1,126}[_a-zA-Z0-9]$', ns.firewall_rule_name):
        raise ValidationError('The firewall rule name can only contain 0-9, a-z, A-Z, \'-\' and \'_\'. '
                              'Additionally, the name of the firewall rule must be at least 3 characters '
                              'and no more than 128 characters in length. ')


def postgres_firewall_rule_name_validator(ns):
    if not ns.firewall_rule_name:
        return
    if not re.search(r'^[a-zA-Z0-9][-_a-zA-Z0-9]{0,79}(?<!-)$', ns.firewall_rule_name):
        raise ValidationError('The firewall rule name can only contain 0-9, a-z, A-Z, \'-\' and \'_\'. '
                              'Additionally, the name of the firewall rule must be at least 1, '
                              'and no more than 80 characters in length. Firewall rule must not end with \'-\'.')


def validate_server_name(db_context, server_name, type_):
    client = db_context.cf_availability(db_context.cmd.cli_ctx, '_')

    if not server_name:
        return

    if len(server_name) < 3 or len(server_name) > 63:
        raise ValidationError('Server name must be at least 3 characters and at most 63 characters.')
    try:
        result = client.check_with_location(db_context.location,
                                            parameters={
                                                'name': server_name,
                                                'type': type_})
    except HttpResponseError as e:
        if e.status_code == 403 and e.error and e.error.code == 'AuthorizationFailed':
            client_without_location = db_context.cf_availability(db_context.cmd.cli_ctx, '_')
            result = client_without_location.check_globally(parameters={'name': server_name, 'type': type_})
        else:
            raise e

    if not result.name_available:
        raise ValidationError(result.message)


def validate_virtual_endpoint_name_availability(cmd, virtual_endpoint_name):
    client = cf_postgres_check_resource_availability(cmd.cli_ctx, '_')
    resource_type = 'Microsoft.DBforPostgreSQL/flexibleServers/virtualendpoints'
    result = client.check_globally(parameters={'name': virtual_endpoint_name, 'type': resource_type})
    if result and result.name_available is False:
        raise ValidationError('Virtual endpoint\'s base name is not available.')


def validate_migration_runtime_server(cmd, migrationInstanceResourceId, target_resource_group_name, target_server_name):
    id_comps = parse_resource_id(migrationInstanceResourceId)
    runtime_server_resource_resource_type = id_comps['resource_type'].lower()
    if 'flexibleservers' != runtime_server_resource_resource_type:
        raise ValidationError('The migration runtime resource identifier must reference a flexible server.')

    server_operations_client = cf_postgres_flexible_servers(cmd.cli_ctx, '_')
    target_server = server_operations_client.get(target_resource_group_name, target_server_name)
    if target_server.id.lower() == migrationInstanceResourceId.lower():
        raise ValidationError('The migration runtime server cannot be the same as the target flexible server. '
                              'Use different values.')


def validate_private_dns_zone(db_context, server_name, private_dns_zone, private_dns_zone_suffix):
    cmd = db_context.cmd
    server_endpoint = cmd.cli_ctx.cloud.suffixes.postgresql_server_endpoint
    if private_dns_zone == server_name + server_endpoint:
        raise ValidationError('The private DNS zone name cannot be the same as the server\'s fully qualified '
                              'domain name.')

    if private_dns_zone[-len(private_dns_zone_suffix):] != private_dns_zone_suffix:
        raise ValidationError("The suffix of the private DNS zone should be '{}'."
                              .format(private_dns_zone_suffix))

    if _is_resource_name(private_dns_zone) and not is_valid_resource_name(private_dns_zone) \
            or not _is_resource_name(private_dns_zone) and not is_valid_resource_id(private_dns_zone):
        raise ValidationError('The private DNS zone name or identifier is not in a valid format.')


# pylint: disable=too-many-branches
def resolve_private_subnet_id(cmd, resource_group_name, vnet, subnet):
    if subnet is not None and vnet is None:
        if not is_valid_resource_id(subnet):
            raise ValidationError('Incorrectly formed subnet identifier. If you are providing '
                                  'only --subnet but not --vnet, the --subnet parameter '
                                  'should be in resource identifier format.')
        parsed_subnet = parse_resource_id(subnet)
        if 'child_name_1' not in parsed_subnet:
            raise ValidationError('Incorrectly formed subnet identifier. Check if the subnet '
                                  'identifier is in the right format.')
        return subnet

    if subnet is None:
        raise RequiredArgumentMissingError('When --vnet is provided, --subnet must also be provided '
                                           'in the form of subnet name.')

    if is_valid_resource_id(vnet):
        if not _is_resource_name(subnet) or not is_valid_resource_name(subnet):
            raise ValidationError('If you pass --vnet as a resource identifier, --subnet '
                                  'must be a subnet name.')
        vnet_subscription, vnet_resource_group, vnet_name, _ = get_id_components(vnet)
        return resource_id(subscription=vnet_subscription,
                           resource_group=vnet_resource_group,
                           namespace='Microsoft.Network',
                           type='virtualNetworks',
                           name=vnet_name,
                           child_type_1='subnets',
                           child_name_1=subnet)

    if _is_resource_name(vnet) and is_valid_resource_name(vnet) and \
            _is_resource_name(subnet) and is_valid_resource_name(subnet):
        return resource_id(subscription=get_subscription_id(cmd.cli_ctx),
                           resource_group=resource_group_name,
                           namespace='Microsoft.Network',
                           type='virtualNetworks',
                           name=vnet,
                           child_type_1='subnets',
                           child_name_1=subnet)

    raise ValidationError('Specify either --subnet as a subnet resource identifier, '
                          '--vnet as a vnet resource identifier with --subnet as a '
                          'subnet name, or both --vnet and --subnet as resource names.')


def _get_private_dns_zone_suffix(cmd, db_context, location, subscription):
    cluster = get_cloud_cluster(cmd, re.sub(r'\s+', '', location).lower(), subscription)

    if cluster is not None:
        private_dns_zone_suffix = cluster['privateDnsZoneDomain']
    else:
        dns_suffix_client = db_context.cf_private_dns_zone_suffix(cmd.cli_ctx, '_')
        private_dns_zone_suffix = dns_suffix_client.get()

    if private_dns_zone_suffix[0] != '.':
        private_dns_zone_suffix = '.' + private_dns_zone_suffix

    return private_dns_zone_suffix


def resolve_private_dns_zone_id(db_context, resource_group_name, server_name, private_dns_zone, subnet_id, location):
    cmd = db_context.cmd
    subscription = get_subscription_id(cmd.cli_ctx)
    dns_resource_group = resource_group_name

    if subnet_id is not None and is_valid_resource_id(subnet_id):
        subscription, dns_resource_group, _, _ = get_id_components(subnet_id)

    private_dns_zone_suffix = _get_private_dns_zone_suffix(cmd, db_context, location, subscription)

    if private_dns_zone is None:
        if 'private' in private_dns_zone_suffix:
            private_dns_zone = server_name + private_dns_zone_suffix
        else:
            private_dns_zone = server_name + '.private' + private_dns_zone_suffix

    validate_private_dns_zone(db_context,
                              server_name=server_name,
                              private_dns_zone=private_dns_zone,
                              private_dns_zone_suffix=private_dns_zone_suffix)

    if is_valid_resource_id(private_dns_zone):
        return private_dns_zone

    return resource_id(subscription=subscription,
                       resource_group=dns_resource_group,
                       namespace='Microsoft.Network',
                       type='privateDnsZones',
                       name=private_dns_zone)


def resolve_public_access_range(public_access, yes):
    if public_access is None:
        try:
            response = get(IP_ADDRESS_CHECKER, timeout=5)
            response.raise_for_status()
            ip_address = response.text.strip()
            if not _validate_ranges_in_ip(ip_address):
                raise ValueError('The detection service returned an invalid IPv4 address.')
        except Exception as ex:
            raise CLIError('Unable to detect your current IP address. Please provide a valid IP address or CIDR '
                           'range for --public-access parameter or set --public-access Disabled. Error: {}'
                           .format(ex))

        logger.warning('Detected current client IP : %s', ip_address)
        if yes:
            return ip_address, ip_address

        if get_user_confirmation('Do you want to enable access to client {0}'.format(ip_address), yes=yes):
            return ip_address, ip_address

        if get_user_confirmation('Do you want to enable access for all IPs', yes=yes):
            return '0.0.0.0', '255.255.255.255'
        return -1, -1

    if str(public_access).lower() == 'all':
        start_ip, end_ip = '0.0.0.0', '255.255.255.255'
    elif str(public_access).lower() in ['none', 'disabled', 'enabled']:
        start_ip, end_ip = -1, -1
    else:
        start_ip, end_ip = parse_public_access_input(public_access)

    return start_ip, end_ip


def build_network_configuration(cmd, resource_group_name, server_name,
                                location, db_context, private_dns_zone_arguments=None, public_access=None,
                                vnet=None, subnet=None, yes=False):
    validate_resource_group(resource_group_name)

    start_ip = -1
    end_ip = -1
    network = postgresql_flexibleservers.models.Network()

    if subnet is not None or vnet is not None:
        if not private_dns_zone_arguments:
            raise RequiredArgumentMissingError('When --vnet or --subnet is provided, --private-dns-zone is required.')
        subnet_id = resolve_private_subnet_id(cmd,
                                              resource_group_name,
                                              vnet=vnet,
                                              subnet=subnet)
        private_dns_zone_id = resolve_private_dns_zone_id(db_context,
                                                          resource_group_name,
                                                          server_name,
                                                          private_dns_zone=private_dns_zone_arguments,
                                                          subnet_id=subnet_id,
                                                          location=location)
        network.delegated_subnet_resource_id = subnet_id
        if private_dns_zone_id is not None:
            network.private_dns_zone_arm_resource_id = private_dns_zone_id
    elif subnet is None and vnet is None and private_dns_zone_arguments is not None:
        raise RequiredArgumentMissingError('Private DNS zone can only be used with private access network. '
                                           'Use --vnet or/and --subnet parameters.')
    else:
        start_ip, end_ip = resolve_public_access_range(public_access, yes=yes)
        if public_access is not None and str(public_access).lower() in ['disabled', 'none']:
            network.public_network_access = 'Disabled'
        else:
            network.public_network_access = 'Enabled'

    return network, start_ip, end_ip


def validate_vnet_location(vnet, location):
    if vnet["location"] != location:
        raise ValidationError('The virtual network must be in the same location as the server.')


def validate_postgres_replica(cmd, tier, location, instance, sku_name,
                              storage_gb, performance_tier=None, list_location_capability_info=None):
    # Tier validation
    if tier == 'Burstable':
        raise ValidationError('Read replica is not supported for the Burstable pricing tier. '
                              'Scale up the source server to General Purpose or Memory Optimized. ')

    if not list_location_capability_info:
        list_location_capability_info = get_postgres_location_capability_info(cmd, location)

    sku_info = list_location_capability_info['sku_info']
    _pg_tier_validator(tier, sku_info)  # need to be validated first
    _pg_sku_name_validator(sku_name, sku_info, tier, instance)
    _pg_storage_performance_tier_validator(performance_tier,
                                           sku_info,
                                           tier,
                                           storage_gb)


def validate_georestore_network(source_server_object, public_access, vnet, subnet, db_engine):
    if source_server_object.network.public_network_access == 'Disabled' and not any((public_access, vnet, subnet)):
        raise ValidationError('Specify network parameters if you are geo-restoring a private access server. '
                              F'Run \'az {db_engine} flexible-server geo-restore --help\' command to see examples.')


def validate_and_format_restore_point_in_time(restore_time):
    try:
        return parser.parse(restore_time)
    except:
        raise ValidationError('The restore point in time value has incorrect date format. '
                              'Use ISO format e.g., 2026-03-22T18:20:22+00:00.')


def is_citus_cluster(cmd, resource_group_name, server_name):
    server_operations_client = cf_postgres_flexible_servers(cmd.cli_ctx, '_')
    server = server_operations_client.get(resource_group_name, server_name)

    return server.cluster and server.cluster.cluster_size > 0


def validate_citus_cluster(cmd, resource_group_name, server_name):
    if is_citus_cluster(cmd, resource_group_name, server_name):
        raise ValidationError('Elastic cluster does not currently support this operation.')


def validate_public_access_server(cmd, resource_group_name, server_name):
    server_operations_client = cf_postgres_flexible_servers(cmd.cli_ctx, '_')

    server = server_operations_client.get(resource_group_name, server_name)
    if server.network.public_network_access == 'Disabled':
        raise ValidationError('Firewall rule operations are not supported for a server '
                              'without public access enabled.')


def _validate_identity(cmd, namespace, identity):
    if is_valid_resource_id(identity):
        return identity

    if _is_resource_name(identity):
        return resource_id(
            subscription=get_subscription_id(cmd.cli_ctx),
            resource_group=namespace.resource_group_name,
            namespace='Microsoft.ManagedIdentity',
            type='userAssignedIdentities',
            name=identity)

    raise ValidationError('Invalid identity name or identifier.')


def validate_identity(cmd, namespace):
    if namespace.identity:
        namespace.identity = _validate_identity(cmd, namespace, namespace.identity)


def validate_byok_identity(cmd, namespace):
    if namespace.byok_identity:
        namespace.byok_identity = _validate_identity(cmd, namespace, namespace.byok_identity)

    if hasattr(namespace, 'backup_byok_identity') and namespace.backup_byok_identity:
        namespace.backup_byok_identity = _validate_identity(cmd, namespace, namespace.backup_byok_identity)


def validate_identities(cmd, namespace):
    if namespace.identities:
        namespace.identities = [_validate_identity(cmd, namespace, identity) for identity in namespace.identities]


def _pg_storage_type_validator(storage_type, auto_grow, performance_tier, tier,
                               supported_storageV2_size, iops, throughput, instance, version):
    is_create_ssdv2 = storage_type and storage_type.lower() == 'premiumv2_lrs'
    is_update_ssdv2 = (instance and instance.storage.type and instance.storage.type.lower() == 'premiumv2_lrs')

    if is_create_ssdv2:
        if version and int(version) < 14:
            raise CLIError('Storage type PremiumV2_LRS is only supported for PostgreSQL version 14 and above.')
        if supported_storageV2_size is None:
            raise CLIError('Invalid value for --storage-type. "PremiumV2_LRS" is not supported for this location.')
        if iops is None or throughput is None:
            raise CLIError('To set --storage-type, required to provide --iops and --throughput.')
    elif instance is None and (throughput is not None or iops is not None):
        raise CLIError('To provide values for both --iops and --throughput, '
                       'set --storage-type to "PremiumV2_LRS".')

    if is_create_ssdv2 or is_update_ssdv2:
        if auto_grow and auto_grow.lower() != 'disabled':
            raise ValidationError('Invalid value for --storage-auto-grow. "Enabled" is not supported for '
                                  'servers with --storage-type set to "PremiumV2_LRS".')
        if performance_tier:
            raise ValidationError('Invalid value for --performance-tier. Performance tier is not supported for '
                                  'servers with --storage-type set to "PremiumV2_LRS".')
        if tier and tier.lower() == 'burstable':
            raise ValidationError('Invalid value for --storage-type. Servers with Burstable tier '
                                  'don\'t support --storage-type set to "PremiumV2_LRS".')
    else:
        if throughput is not None:
            raise CLIError('Invalid value for --throughput. Updating throughput is only supported for '
                           'servers created with --storage-type set to "PremiumV2_LRS".')
        if iops is not None:
            raise CLIError('Invalid value for --iops. Updating storage iops is only supported for '
                           'servers created with --storage-type set to "PremiumV2_LRS".')


def pg_restore_validator(compute_tier, **args):
    is_ssdv2_enabled = args.get('storage_type', None) is not None \
        and args.get('storage_type').lower() == 'premiumv2_lrs'

    if is_ssdv2_enabled and compute_tier.lower() == 'burstable':
        raise ValidationError('Invalid value for --tier. Burstable tier is not supported for servers with '
                              '--storage-type set to "PremiumV2_LRS".')


def _pg_authentication_validator(password_auth, is_microsoft_entra_auth_enabled,
                                 admin_name, admin_id, admin_type, instance):
    if instance is None:
        if (password_auth is not None and password_auth.lower() == 'disabled') and not is_microsoft_entra_auth_enabled:
            raise CLIError('At least one authentication method must be enabled. '
                           'Set --microsoft-entra-auth and/or --password-auth to "Enabled".')

        if not is_microsoft_entra_auth_enabled and (admin_name or admin_id or admin_type):
            raise CLIError('To provide values for --admin-object-id, --admin-display-name, and --admin-type, '
                           'set --microsoft-entra-auth to "Enabled".')
        if (admin_name is not None or admin_id is not None or admin_type is not None) and \
           not (admin_name is not None and admin_id is not None and admin_type is not None):
            raise CLIError('To add a Microsoft Entra admin, provide values for --admin-object-id, '
                           '--admin-display-name, and --admin-type.')


def check_resource_group(resource_group_name):
    # check if rg is already null originally
    if not resource_group_name:
        return False

    # replace single and double quotes with empty string
    resource_group_name = resource_group_name.replace("'", '')
    resource_group_name = resource_group_name.replace('"', '')

    # check if rg is empty after removing quotes
    if not resource_group_name:
        return False
    return True


def validate_resource_group(resource_group_name):
    if not check_resource_group(resource_group_name):
        raise CLIError('Resource group name cannot be empty.')


def validate_backup_name(backup_name):
    # check if backup_name is already null originally
    if not backup_name:
        raise CLIError('Backup name cannot be empty.')

    # replace single and double quotes with empty string
    backup_name = backup_name.replace("'", '')
    backup_name = backup_name.replace('"', '')

    # check if backup_name is empty or contains only whitespace after removing the quote
    if not backup_name or backup_name.isspace():
        raise CLIError('Backup name cannot be empty or contain only whitespace.')

    # check if backup_name exceeds 128 characters
    if len(backup_name) > 128:
        raise CLIError('Backup name cannot exceed 128 characters.')


def validate_database_name(database_name):
    if database_name is not None and not re.match(r'^[a-zA-Z_][\w\-]{0,62}$', database_name):
        raise ValidationError('Database name must begin with a letter (a-z) or underscore (_). '
                              'Subsequent characters in a name can be letters, digits (0-9), hyphens (-), '
                              'or underscores. Database name length must be less than 64 characters.')



# --------------------------------------------------------------------------------------------
# Copyright (c) Microsoft Corporation. All rights reserved.
# Licensed under the MIT License. See License.txt in the project root for license information.
# --------------------------------------------------------------------------------------------

import unittest
import os

from azure.cli.core.commands.validators import validate_file_or_dict, JSON_RECOMMENDATION_MESSAGE
from azure.cli.core.azclierror import InvalidArgumentValueError


class TestValidators(unittest.TestCase):

    def test_validate_file_or_dict(self):
        json_str = '{"name": "my-resource"}'
        curr_dir = os.path.dirname(os.path.realpath(__file__))
        temp_file = os.path.join(curr_dir, 'temp.json')

        # Parse JSON file
        with open(temp_file, 'w') as f:
            f.write(json_str)
        json_read = validate_file_or_dict(temp_file)
        assert json_read['name'] == 'my-resource'
        os.remove(temp_file)

        # Parse in-line JSON string
        json_read = validate_file_or_dict(json_str)
        assert json_read['name'] == 'my-resource'

        # Test error 1: Parse JSON file with error
        with open(temp_file, 'w') as f:
            f.write(json_str)
            f.write('error!')
        with self.assertRaisesRegex(InvalidArgumentValueError, 'Failed to parse file') as ex:
            validate_file_or_dict(temp_file)
        assert ex.exception.recommendations[0] == JSON_RECOMMENDATION_MESSAGE
        assert len(ex.exception.recommendations) == 1
        os.remove(temp_file)

        # Test error 2: A non-existing file, but it looks like a JSON file
        with self.assertRaisesRegex(InvalidArgumentValueError, 'JSON file does not exist') as ex:
            validate_file_or_dict("not-exist.json")
        assert ex.exception.recommendations[0] == JSON_RECOMMENDATION_MESSAGE
        assert len(ex.exception.recommendations) == 1

        # Test error 3: A non-existing file, or invalid JSON string
        with self.assertRaisesRegex(InvalidArgumentValueError, 'Failed to parse string as JSON') as ex:
            validate_file_or_dict("invalid-string")
        assert ex.exception.recommendations[0] == JSON_RECOMMENDATION_MESSAGE
        assert 'The provided JSON string may have been parsed by the shell.' in ex.exception.recommendations[1]


if __name__ == '__main__':
    unittest.main()



# --------------------------------------------------------------------------------------------
# Copyright (c) Microsoft Corporation. All rights reserved.
# Licensed under the MIT License. See License.txt in the project root for license information.
# --------------------------------------------------------------------------------------------


class AzureTestError(Exception):
    def __init__(self, error_message):
        message = 'An error caused by the Azure test harness failed the test: {}'
        super().__init__(message.format(error_message))



# --------------------------------------------------------------------------------------------
# Copyright (c) Microsoft Corporation. All rights reserved.
# Licensed under the MIT License. See License.txt in the project root for license information.
# --------------------------------------------------------------------------------------------
import os
import unittest

from azure.cli.testsdk import ScenarioTest, ResourceGroupPreparer
from azure.cli.testsdk.scenario_tests import AllowLargeResponse
from knack.util import CLIError


class CognitiveServicesCommandTests(ScenarioTest):
    @ResourceGroupPreparer()
    def test_cognitiveservices_crud(self, resource_group):
        sname = self.create_random_name(prefix='cog', length=12)
        tagname = self.create_random_name(prefix='tagname', length=15)
        tagvalue = self.create_random_name(prefix='tagvalue', length=15)

        self.kwargs.update({
            'sname': sname,
            'kind': 'FormRecognizer',
            'sku': 'S0',
            'location': 'SOUTHCENTRALUS',
            'tags': tagname + '=' + tagvalue
        })

        # test to create cognitive services account
        self.cmd('az cognitiveservices account create -n {sname} -g {rg} --kind {kind} --sku {sku} -l {location} --yes',
                 checks=[self.check('name', '{sname}'),
                         self.check('location', '{location}'),
                         self.check('sku.name', '{sku}')])

        # test to show the details of cognitive services account
        self.cmd('az cognitiveservices account show -n {sname} -g {rg}',
                 checks=[self.check('name', '{sname}'),
                         self.check('resourceGroup', '{rg}')])

        # test to update the properties of cognitive services account
        self.cmd('az cognitiveservices account update -n {sname} -g {rg} --sku {sku} --tags {tags}',
                 checks=[self.check('sku.name', '{sku}'),
                         self.check('tags', {tagname: tagvalue})])

        # test to list keys of a cogntive services account
        oldkeys = self.cmd('az cognitiveservices account keys list -n {sname} -g {rg}',
                           checks=[self.check('length(key1)', 84),
                                   self.check('length(key2)', 84)]).get_output_in_json()

        # test to regenerate the keys of a cognitive services account
        newkeys = self.cmd('az cognitiveservices account keys regenerate -n {sname} -g {rg} --key-name Key1').get_output_in_json()  # pylint: disable=line-too-long
        self.assertNotEqual(oldkeys, newkeys)

        # test to list cognitive service accounts under current resource group
        self.cmd('az cognitiveservices account list -g {rg}', checks=[
            self.check('length(@)', 1)])

        # test to delete the cognitive services account
        exitcode = self.cmd('az cognitiveservices account delete -n {sname} -g {rg}').exit_code
        self.assertEqual(exitcode, 0)

    @AllowLargeResponse()
    @ResourceGroupPreparer()
    def test_cognitiveservices_account_list_kinds(self, resource_group):
        # test to list cognitive services account kinds
        results = self.cmd('az cognitiveservices account list-kinds').get_output_in_json()
        self.assertTrue(len(results) > 0)
        self.assertTrue('Face' in results)

    @ResourceGroupPreparer()
    def test_cognitiveservices_account_list_skus_legacy(self, resource_group):

        self.kwargs.update({
            'name': self.create_random_name(prefix='cs_cli_test_', length=16),
            'kind': 'FormRecognizer',
            'sku': 'S0',
            'location': 'SOUTHCENTRALUS'
        })

        self.cmd('az cognitiveservices account create -n {name} -g {rg} --kind {kind} --sku {sku} -l {location} --yes',
                 checks=[self.check('name', '{name}'),
                         self.check('location', '{location}'),
                         self.check('sku.name', '{sku}')])

        results = self.cmd('az cognitiveservices account list-skus -n {name} -g {rg}').get_output_in_json()
        self.assertTrue(isinstance(results['value'], list))
        self.assertTrue(len(results['value']) > 0)

    @AllowLargeResponse()
    @ResourceGroupPreparer()
    def test_cognitiveservices_account_list_skus(self, resource_group):

        self.kwargs.update({
            'kind': 'Face',
            'location': 'westus'
        })

        results = self.cmd('az cognitiveservices account list-skus --kind {kind}').get_output_in_json()
        self.assertTrue(isinstance(results, list))
        self.assertTrue(len(results) > 0)

        for sku in results:
            self.assertTrue(sku['kind'] == self.kwargs['kind'])

        results = self.cmd('az cognitiveservices account list-skus --kind {kind} --location {location}').get_output_in_json()
        self.assertTrue(isinstance(results, list))
        self.assertTrue(len(results) > 0)

        for sku in results:
            self.assertTrue(sku['kind'] == self.kwargs['kind'])
            self.assertTrue(self.kwargs['location'].lower() in [x.lower() for x in sku['locations']])

    @ResourceGroupPreparer()
    def test_cognitiveservices_account_list_usage(self, resource_group):

        self.kwargs.update({
            'name': self.create_random_name(prefix='cs_cli_test_', length=16),
            'kind': 'TextAnalytics',
            'sku': 'S',
            'location': 'SOUTHCENTRALUS'
        })

        self.cmd('az cognitiveservices account create -n {name} -g {rg} --kind {kind} --sku {sku} -l {location}',
                 checks=[self.check('name', '{name}'),
                         self.check('location', '{location}'),
                         self.check('sku.name', '{sku}')])

        results = self.cmd('az cognitiveservices account list-usage -n {name} -g {rg}').get_output_in_json()
        self.assertTrue(isinstance(results, list))


if __name__ == '__main__':
    unittest.main()



# --------------------------------------------------------------------------------------------
# Copyright (c) Microsoft Corporation. All rights reserved.
# Licensed under the MIT License. See License.txt in the project root for license information.
# --------------------------------------------------------------------------------------------

from azure.cli.testsdk.scenario_tests import AllowLargeResponse
from azure.cli.testsdk import (
    JMESPathCheck,
    NoneCheck,
    ResourceGroupPreparer,
    ScenarioTest)
from .server_preparer import ServerPreparer
from .constants import DEFAULT_LOCATION


class FlexibleServerDatabaseMgmtScenarioTest(ScenarioTest):

    postgres_location = DEFAULT_LOCATION

    @AllowLargeResponse()
    @ResourceGroupPreparer(location=postgres_location)
    @ServerPreparer(location=postgres_location)
    def test_postgres_flexible_server_database_mgmt(self, resource_group, server):
        self._test_database_mgmt(resource_group, server)

    def _test_database_mgmt(self, resource_group, server):

        database_name = self.create_random_name('database', 20)

        self.cmd('postgres flexible-server db create -g {} -s {} -n {}'.format(resource_group, server, database_name),
                 checks=[JMESPathCheck('name', database_name)])

        self.cmd('postgres flexible-server db show -g {} -s {} -n {}'.format(resource_group, server, database_name),
                 checks=[
                     JMESPathCheck('name', database_name),
                     JMESPathCheck('resourceGroup', resource_group)])

        self.cmd('postgres flexible-server db list -g {} -s {} '.format(resource_group, server),
                 checks=[JMESPathCheck('type(@)', 'array')])

        self.cmd('postgres flexible-server db delete -g {} -s {} -n {} --yes'.format(resource_group, server, database_name),
                 checks=NoneCheck())


# --------------------------------------------------------------------------------------------
# Copyright (c) Microsoft Corporation. All rights reserved.
# Licensed under the MIT License. See License.txt in the project root for license information.
# --------------------------------------------------------------------------------------------

import argparse
import base64
import binascii
from datetime import datetime, timezone
import re
import sys
from ipaddress import ip_network

from enum import Enum
from knack.deprecation import Deprecated
from knack.log import get_logger
from knack.util import CLIError

from azure.cli.core.commands.client_factory import get_mgmt_service_client
from azure.cli.core.commands.validators import validate_tags
from azure.cli.core.azclierror import RequiredArgumentMissingError, InvalidArgumentValueError, AzureInternalError
from azure.cli.core.profiles import ResourceType
from azure.cli.core.util import get_file_json, shell_safe_json_parse

logger = get_logger(__name__)

secret_text_encoding_values = ['utf-8', 'utf-16le', 'utf-16be', 'ascii']
secret_binary_encoding_values = ['base64', 'hex']


class KeyEncryptionDataType(str, Enum):
    BASE64 = 'base64'
    PLAINTEXT = 'plaintext'


def _extract_version(item_id):
    return item_id.split('/')[-1]


def _get_resource_group_from_resource_name(cli_ctx, vault_name, hsm_name=None):
    """
    Fetch resource group from vault name
    :param str vault_name: name of the key vault
    :param str hsm_name: name of the managed hsm
    :return: resource group name or None
    :rtype: str
    """
    from azure.mgmt.core.tools import parse_resource_id

    if vault_name:
        client = get_mgmt_service_client(cli_ctx, ResourceType.MGMT_KEYVAULT).vaults
        for vault in client.list(filter=f"resourceType eq 'Microsoft.KeyVault/vaults' and name eq '{vault_name}'"):
            id_comps = parse_resource_id(vault.id)
            if id_comps.get('name', None) and id_comps['name'].lower() == vault_name.lower():
                return id_comps['resource_group']

    if hsm_name:
        client = get_mgmt_service_client(cli_ctx, ResourceType.MGMT_KEYVAULT).managed_hsms
        try:
            for hsm in client.list_by_subscription():
                id_comps = parse_resource_id(hsm.id)
                if id_comps.get('name', None) and id_comps['name'].lower() == hsm_name.lower():
                    return id_comps['resource_group']
        except:  # pylint: disable=bare-except
            return None

    return None


# COMMAND NAMESPACE VALIDATORS
def process_secret_set_namespace(namespace):
    validate_tags(namespace)

    content = namespace.value
    file_path = namespace.file_path
    encoding = namespace.encoding
    tags = namespace.tags or {}

    use_error = CLIError("incorrect usage: [Required] --value VALUE | --file PATH")

    if (content and file_path) or (not content and not file_path):
        raise use_error

    namespace.enabled = not namespace.disabled if namespace.disabled is not None else None
    del namespace.file_path
    del namespace.encoding
    del namespace.disabled

    encoding = encoding or 'utf-8'
    if file_path:
        if encoding in secret_text_encoding_values:
            with open(file_path, 'r') as f:
                try:
                    content = f.read()
                except UnicodeDecodeError:
                    raise CLIError("Unable to decode file '{}' with '{}' encoding.".format(
                        file_path, encoding))
                encoded_str = content
                encoded = content.encode(encoding)
                decoded = encoded.decode(encoding)
        elif encoding == 'base64':
            with open(file_path, 'rb') as f:
                content = f.read()
                try:
                    encoded = base64.encodebytes(content)
                except AttributeError:
                    encoded = base64.encodestring(content)  # pylint: disable=deprecated-method
                encoded_str = encoded.decode('utf-8')
                decoded = base64.b64decode(encoded_str)
        elif encoding == 'hex':
            with open(file_path, 'rb') as f:
                content = f.read()
                encoded = binascii.b2a_hex(content)
                encoded_str = encoded.decode('utf-8')
                decoded = binascii.unhexlify(encoded_str)

        if content != decoded:
            raise CLIError("invalid encoding '{}'".format(encoding))

        content = encoded_str

    tags.update({'file-encoding': encoding})
    namespace.tags = tags
    namespace.value = content


def process_hsm_name(ns):
    if not ns.identifier and not ns.hsm_name:
        raise CLIError('Please specify --hsm-name or --id.')
    if ns.identifier:
        ns.hsm_name = ns.identifier


def validate_vault_name_and_hsm_name(ns):
    vault_name = getattr(ns, 'vault_name', None)
    hsm_name = getattr(ns, 'hsm_name', None)
    if vault_name and hsm_name:
        raise CLIError('--vault-name and --hsm-name are mutually exclusive.')

    if not vault_name and not hsm_name:
        raise CLIError('Please specify --vault-name or --hsm-name.')

# PARAMETER NAMESPACE VALIDATORS


def validate_retention_days_on_creation(ns):
    # If customer has specified retention days, nothing to do
    if ns.retention_days:
        return
    # If customer has not specified retention days,
    # ask for mandatory retention days for MHSM creation, else set to default '90' for keyvault creation
    if getattr(ns, 'hsm_name', None):
        raise RequiredArgumentMissingError("--retention-days is required for MHSM creation.")
    if getattr(ns, 'vault_name', None):
        ns.retention_days = '90'


def get_attribute_validator(name, attribute_class, create=False):
    def validator(ns):
        ns_dict = ns.__dict__
        enabled = not ns_dict.pop('disabled') if create else ns_dict.pop('enabled')
        attributes = attribute_class(
            enabled=enabled,
            not_before=ns_dict.pop('not_before', None),
            expires=ns_dict.pop('expires', None))
        setattr(ns, '{}_attributes'.format(name), attributes)

    return validator


def validate_key_import_source(ns):
    byok_file = ns.byok_file
    byok_string = ns.byok_string
    pem_file = ns.pem_file
    pem_string = ns.pem_string
    pem_password = ns.pem_password
    if len([arg for arg in [byok_file, byok_string, pem_file, pem_string] if arg]) != 1:
        raise ValueError('supply exactly one: --byok-file, --byok-string, --pem-file, --pem-string')
    if (byok_file or byok_string) and pem_password:
        raise ValueError('--byok-file or --byok-string cannot be used with --pem-password')
    if pem_password and not pem_file and not pem_string:
        raise ValueError('--pem-password must be used with --pem-file or --pem-string')


def validate_key_import_type(ns):
    # Default value of kty is: RSA
    kty = getattr(ns, 'kty', None)
    crv = getattr(ns, 'curve', None)

    if (kty == 'EC' and crv is None) or (kty != 'EC' and crv):
        from azure.cli.core.azclierror import ValidationError
        raise ValidationError('parameter --curve should be specified when key type --kty is EC.')


def validate_key_type(ns):
    crv = getattr(ns, 'curve', None)
    kty = getattr(ns, 'kty', None) or ('EC' if crv else 'RSA')
    protection = getattr(ns, 'protection', None)

    if protection == 'hsm':
        kty = kty if kty.endswith('-HSM') else kty + '-HSM'
    elif protection == 'software':
        if getattr(ns, 'byok_file', None):
            raise CLIError('BYOK keys are hardware protected. Omit --protection')
        if kty.endswith('-HSM'):
            raise CLIError('The key type {} is invalid for software protected keys. Omit --protection')

    setattr(ns, 'kty', kty)


# pylint: disable=line-too-long
def _fetch_default_release_policy(cli_ctx, vault_url, policy_type='cvm'):
    try:
        # get vault/hsm location
        mgmt_client = get_mgmt_service_client(cli_ctx, ResourceType.MGMT_KEYVAULT)
        location = None
        parsed_vault_url = vault_url.removeprefix('https://').split('.')
        if parsed_vault_url[1] == 'vault':
            vault_name_filter = f"resourceType eq 'Microsoft.KeyVault/vaults' and name eq '{parsed_vault_url[0]}'"
            for vault in mgmt_client.vaults.list(filter=vault_name_filter):
                location = vault.location
                break
        elif parsed_vault_url[1] == 'managedhsm':
            for hsm in mgmt_client.managed_hsms.list_by_subscription():
                if hsm.name == parsed_vault_url[0]:
                    location = hsm.location
                    break
        if not location:
            raise InvalidArgumentValueError(f"Fail to fetch default cvm policy due to invalid {vault_url}")

        # call MAA to get default cvm policy
        from azure.cli.core.util import send_raw_request
        from azure.cli.core.commands.client_factory import get_subscription_id
        _endpoint = cli_ctx.cloud.endpoints.resource_manager
        if _endpoint.endswith('/'):
            _endpoint = _endpoint[:-1]
        default_release_policy_url = f"{_endpoint}/subscriptions/{get_subscription_id(cli_ctx)}/providers/Microsoft.Attestation/Locations/{location}/defaultProvider?api-version=2020-10-01"
        response = send_raw_request(cli_ctx, 'get', default_release_policy_url)
        if response.status_code != 200:
            raise AzureInternalError(f"Fail to fetch default release policy from {default_release_policy_url}")

        # extract attest uri from response as authority in cvm policy
        import json
        res_json = json.loads(response.text)
        attest_uri = res_json['properties']['attestUri']
        if policy_type == 'cvm':
            default_release_policy = {
                'version': '1.0.0',
                'anyOf': [
                    {
                        'authority': attest_uri,
                        'allOf': [
                            {
                                'claim': 'x-ms-compliance-status',
                                'equals': 'azure-compliant-cvm'
                            }
                        ]
                    }
                ]
            }
        else:
            default_release_policy = {
                'version': '1.0.0',
                'anyOf': [
                    {
                        'authority': attest_uri,
                        'allOf': [
                            {
                                'anyOf': [
                                    {
                                        'claim': 'x-ms-isolation-tee.x-ms-attestation-type',
                                        'equals': 'sevsnpvm'
                                    },
                                    {
                                        'claim': 'x-ms-isolation-tee.x-ms-attestation-type',
                                        'equals': 'tdxvm'
                                    }
                                ]
                            },
                            {
                                'claim': 'x-ms-isolation-tee.x-ms-compliance-status',
                                'equals': 'azure-compliant-cvm'
                            }
                        ]
                    }
                ]
            }
        return default_release_policy
    except Exception as ex:  # pylint: disable=broad-except
        raise AzureInternalError(f"Fail to fetch default release policy: {ex}")


def process_key_release_policy(cmd, ns):
    default_cvm_policy = None
    default_data_disk_policy = None
    if hasattr(ns, 'default_cvm_policy'):
        default_cvm_policy = ns.default_cvm_policy
        del ns.default_cvm_policy
    if hasattr(ns, 'default_data_disk_policy'):
        default_data_disk_policy = ns.default_data_disk_policy
        del ns.default_data_disk_policy

    immutable = None
    if hasattr(ns, 'immutable'):
        immutable = ns.immutable
        del ns.immutable

    if not ns.release_policy and not default_cvm_policy and not default_data_disk_policy:
        if immutable is not None:
            raise InvalidArgumentValueError('Please provide policy when setting `--immutable`')
        return

    if ns.release_policy and default_cvm_policy:
        raise InvalidArgumentValueError('Can not specify both `--policy` and `--default-cvm-policy`')
    if ns.release_policy and default_data_disk_policy:
        raise InvalidArgumentValueError('Can not specify both `--policy` and `--default-data-disk-policy`')
    if default_cvm_policy and default_data_disk_policy:
        from azure.cli.core.azclierror import MutuallyExclusiveArgumentError
        raise MutuallyExclusiveArgumentError('`--default-cvm-policy` and `--default-data-disk-policy` '
                                             'are mutually exclusive')

    import json
    KeyReleasePolicy = cmd.loader.get_sdk('KeyReleasePolicy', mod='_models',
                                          resource_type=ResourceType.DATA_KEYVAULT_KEYS)
    if default_cvm_policy or default_data_disk_policy:
        vault_url = getattr(ns, 'hsm_name', None) or getattr(ns, 'vault_base_url', None)
        if not vault_url:
            vault_url = getattr(ns, 'identifier', None)
        policy = _fetch_default_release_policy(cmd.cli_ctx, vault_url, 'cvm' if default_cvm_policy else 'data_disk')
        ns.release_policy = KeyReleasePolicy(encoded_policy=json.dumps(policy).encode('utf-8'),
                                             immutable=immutable)
        return

    import os
    if os.path.exists(ns.release_policy):
        data = get_file_json(ns.release_policy)
    else:
        data = shell_safe_json_parse(ns.release_policy)

    ns.release_policy = KeyReleasePolicy(encoded_policy=json.dumps(data).encode('utf-8'),
                                         immutable=immutable)


def validate_policy_permissions(ns):
    key_perms = ns.key_permissions
    secret_perms = ns.secret_permissions
    cert_perms = ns.certificate_permissions
    storage_perms = ns.storage_permissions

    if key_perms is None and secret_perms is None and cert_perms is None and storage_perms is None:
        raise argparse.ArgumentError(
            None,
            'specify at least one: --key-permissions, --secret-permissions, '
            '--certificate-permissions --storage-permissions')


def validate_private_endpoint_connection_id(cmd, ns):
    if ns.connection_id:
        from azure.cli.core.util import parse_proxy_resource_id
        result = parse_proxy_resource_id(ns.connection_id)
        ns.resource_group_name = result['resource_group']
        if result['type'] and 'managedHSM' in result['type']:
            ns.hsm_name = result['name']
        else:
            ns.vault_name = result['name']
        ns.private_endpoint_connection_name = result['child_name_1']

    if not ns.resource_group_name:
        ns.resource_group_name = _get_resource_group_from_resource_name(cli_ctx=cmd.cli_ctx,
                                                                        vault_name=getattr(ns, 'vault_name', None),
                                                                        hsm_name=getattr(ns, 'hsm_name', None))

    if not all([(getattr(ns, 'vault_name', None) or getattr(ns, 'hsm_name', None)),
                ns.resource_group_name, ns.private_endpoint_connection_name]):
        raise CLIError('incorrect usage: [--id ID | --name NAME --vault-name NAME | --name NAME --hsm-name NAME]')

    del ns.connection_id


def validate_principal(ns):
    num_set = sum(1 for p in [ns.object_id, ns.spn, ns.upn] if p)
    if num_set != 1:
        raise argparse.ArgumentError(
            None, 'specify exactly one: --object-id, --spn, --upn')


def validate_resource_group_name(cmd, ns):
    """
    Populate resource_group_name, if not provided
    """
    if 'keyvault purge' in cmd.name or 'keyvault recover' in cmd.name:
        return

    vault_name = getattr(ns, 'vault_name', None)
    hsm_name = getattr(ns, 'hsm_name', None)
    if 'keyvault update-hsm' in cmd.name or 'keyvault region' in cmd.name:
        hsm_name = getattr(ns, 'name', None)

    if vault_name and hsm_name:
        raise CLIError('--name/-n and --hsm-name are mutually exclusive.')

    if vault_name:
        # This is a temporary solution for showing deprecation message only for vaults
        _show_vault_only_deprecate_message(ns)

    if not ns.resource_group_name:
        group_name = _get_resource_group_from_resource_name(cmd.cli_ctx, vault_name, hsm_name)
        if group_name:
            ns.resource_group_name = group_name
        else:
            if vault_name:
                resource_type = 'Vault'
            else:
                resource_type = 'HSM'
            msg = "The {} '{}' not found within subscription."
            raise CLIError(msg.format(resource_type, vault_name if vault_name else hsm_name))


def validate_deleted_vault_or_hsm_name(cmd, ns):
    """
    Validate a deleted vault name; populate or validate location and resource_group_name
    """
    from azure.mgmt.core.tools import parse_resource_id

    vault_name = getattr(ns, 'vault_name', None)
    hsm_name = getattr(ns, 'hsm_name', None)

    if not vault_name and not hsm_name:
        raise CLIError('Please specify --vault-name or --hsm-name.')

    if vault_name:
        resource_name = vault_name
    else:
        resource_name = hsm_name
    resource = None

    if vault_name:
        client = get_mgmt_service_client(cmd.cli_ctx, ResourceType.MGMT_KEYVAULT).vaults
    else:
        client = get_mgmt_service_client(cmd.cli_ctx, ResourceType.MGMT_KEYVAULT).managed_hsms

    # if the location is specified, use get_deleted rather than list_deleted
    if ns.location:
        resource = client.get_deleted(resource_name, ns.location)
        if vault_name:
            id_comps = parse_resource_id(resource.properties.vault_id)
        else:
            id_comps = parse_resource_id(resource.properties.mhsm_id)

    # otherwise, iterate through deleted vaults to find one with a matching name
    else:
        for v in client.list_deleted():
            if vault_name:
                id_comps = parse_resource_id(v.properties.vault_id)
            else:
                id_comps = parse_resource_id(v.properties.mhsm_id)
            if id_comps['name'].lower() == resource_name.lower():
                resource = v
                ns.location = resource.properties.location
                break

    # if the vault was not found, throw an error
    if not resource:
        raise CLIError('No deleted Vault or HSM was found with name ' + resource_name)

    if 'keyvault purge' not in cmd.name and 'keyvault show-deleted' not in cmd.name:
        setattr(ns, 'resource_group_name', getattr(ns, 'resource_group_name', None) or id_comps['resource_group'])

        # resource_group_name must match the resource group of the deleted vault
        if id_comps['resource_group'] != ns.resource_group_name:
            raise CLIError("The specified resource group does not match that of the deleted vault or hsm %s. The vault "
                           "or hsm must be recovered to the original resource group %s."
                           % (vault_name, id_comps['resource_group']))


def validate_x509_certificate_chain(ns):
    def _load_certificate_as_bytes(file_name):
        cert_list = []
        regex = r'-----BEGIN CERTIFICATE-----([^-]+)-----END CERTIFICATE-----'
        with open(file_name, 'r') as f:
            cert_data = f.read()
            for entry in re.findall(regex, cert_data):
                cert_list.append(base64.b64decode(entry.replace('\n', '')))
        return cert_list

    ns.x509_certificates = _load_certificate_as_bytes(ns.x509_certificates)


# ARGUMENT TYPES


def certificate_type(string):
    """ Loads file and outputs contents as base64 encoded string. """
    import os
    try:
        with open(os.path.expanduser(string), 'rb') as f:
            cert_data = f.read()
        return cert_data
    except OSError as e:
        raise CLIError("Unable to load certificate file '{}': {}.".format(string, e.strerror))


def datetime_type(string):
    """ Validates UTC datettime in accepted format. Examples: 2017-12-31T01:11:59Z,
    2017-12-31T01:11Z or 2017-12-31T01Z or 2017-12-31 """
    accepted_date_formats = ['%Y-%m-%dT%H:%M:%SZ', '%Y-%m-%dT%H:%MZ',
                             '%Y-%m-%dT%HZ', '%Y-%m-%d']
    for form in accepted_date_formats:
        try:
            return datetime.strptime(string, form).replace(tzinfo=timezone.utc)
        except ValueError:  # checks next format
            pass
    raise ValueError("Input '{}' not valid. Valid example: 2000-12-31T12:59:59Z".format(string))


def _get_base_url_type(cli_ctx, service):
    suffix = ''
    if service == 'vault':
        suffix = cli_ctx.cloud.suffixes.keyvault_dns
    elif service == 'hsm':
        from azure.cli.core.cloud import CloudSuffixNotSetException
        try:
            suffix = cli_ctx.cloud.suffixes.mhsm_dns
        except CloudSuffixNotSetException:  # For Azure Stack and Air-Gaped Cloud
            suffix = ''

    def base_url_type(name):
        return 'https://{}{}'.format(name, suffix)

    return base_url_type


def get_vault_base_url_type(cli_ctx):
    return _get_base_url_type(cli_ctx, service='vault')


def get_hsm_base_url_type(cli_ctx):
    return _get_base_url_type(cli_ctx, service='hsm')


def _construct_vnet(cmd, resource_group_name, vnet_name, subnet_name):
    from azure.mgmt.core.tools import resource_id
    from azure.cli.core.commands.client_factory import get_subscription_id

    return resource_id(
        subscription=get_subscription_id(cmd.cli_ctx),
        resource_group=resource_group_name,
        namespace='Microsoft.Network',
        type='virtualNetworks',
        name=vnet_name,
        child_type_1='subnets',
        child_name_1=subnet_name)


def validate_subnet(cmd, namespace):
    from azure.mgmt.core.tools import is_valid_resource_id

    subnet = namespace.subnet
    subnet_is_id = is_valid_resource_id(subnet)
    vnet = namespace.vnet_name

    if (subnet_is_id and not vnet) or (not subnet and not vnet):
        return

    if subnet and not subnet_is_id and vnet:
        namespace.subnet = _construct_vnet(cmd, namespace.resource_group_name, vnet, subnet)
    else:
        raise CLIError('incorrect usage: [--subnet ID | --subnet NAME --vnet-name NAME]')


def validate_ip_address(namespace):
    # if there are overlapping ip ranges, throw an exception
    ip_address = namespace.ip_address

    if not ip_address:
        return

    ip_address_networks = [ip_network(ip) for ip in ip_address]
    for idx, ip_address_network in enumerate(ip_address_networks):
        for idx2, ip_address_network2 in enumerate(ip_address_networks):
            if idx == idx2:
                continue
            if ip_address_network.overlaps(ip_address_network2):
                raise InvalidArgumentValueError(f"ip addresses {ip_address_network} and {ip_address_network2} "
                                                f"provided are overlapping: --ip_address ip1 [ip2]...")


def validate_role_assignment_args(ns):
    if not any([ns.role_assignment_name, ns.scope, ns.assignee, ns.assignee_object_id, ns.role, ns.ids]):
        raise RequiredArgumentMissingError(
            'Please specify at least one of these parameters: '
            '--name, --scope, --assignee, --assignee-object-id, --role, --ids')


def validate_vault_or_hsm(ns):
    identifier = getattr(ns, 'identifier', None)
    vault_base_url = getattr(ns, 'vault_base_url', None)
    hsm_name = getattr(ns, 'hsm_name', None)
    if identifier:
        if vault_base_url:
            raise CLIError('--vault-name and --id are mutually exclusive.')
        if hsm_name:
            raise CLIError('--hsm-name and --id are mutually exclusive.')

        items = identifier.split('/')
        if len(items) < 3:
            raise CLIError('Invalid id for Vault or HSM.')

        ns.vault_base_url = ns.identifier = '/'.join(items[:3])
    else:
        if vault_base_url and hsm_name:
            raise CLIError('--vault-name and --hsm-name are mutually exclusive.')

        if not vault_base_url and not hsm_name:
            raise CLIError('Please specify --vault-name or --hsm-name.')


def _show_vault_only_deprecate_message(ns):
    message_dict = {
        'keyvault delete':
            Deprecated(ns.cmd.cli_ctx, message_func=lambda x:
                       'Warning! If you have soft-delete protection enabled on this key vault, you will '
                       'not be able to reuse this key vault name until the key vault has been purged from '
                       'the soft deleted state. Please see the following documentation for additional '
                       'guidance.\nhttps://learn.microsoft.com/azure/key-vault/general/soft-delete-overview'),
        'keyvault key delete':
            Deprecated(ns.cmd.cli_ctx, message_func=lambda x:
                       'Warning! If you have soft-delete protection enabled on this key vault, this key '
                       'will be moved to the soft deleted state. You will not be able to create a key with '
                       'the same name within this key vault until the key has been purged from the '
                       'soft-deleted state. Please see the following documentation for additional '
                       'guidance.\nhttps://learn.microsoft.com/azure/key-vault/general/soft-delete-overview')
    }
    cmds = ['keyvault delete', 'keyvault key delete']
    for cmd in cmds:
        if cmd == getattr(ns, 'command', None):
            print(message_dict[cmd].message, file=sys.stderr)


def set_vault_base_url(ns):
    vault_base_url = getattr(ns, 'vault_base_url', None)
    hsm_name = getattr(ns, 'hsm_name', None)

    if not hsm_name:
        # This is a temporary solution for showing deprecation message only for vaults
        _show_vault_only_deprecate_message(ns)

    if hsm_name and not vault_base_url:
        setattr(ns, 'vault_base_url', hsm_name)


def validate_key_id(entity_type):
    def _validate(ns):
        from azure.keyvault.keys._shared import parse_key_vault_id

        pure_entity_type = entity_type.replace('deleted', '')
        name = getattr(ns, pure_entity_type + '_name', None)
        vault = getattr(ns, 'vault_base_url', None)
        if not vault:
            vault = getattr(ns, 'hsm_name', None)
        identifier = getattr(ns, 'identifier', None)

        if identifier:
            vault_base_url = getattr(ns, 'vault_base_url', None)
            hsm_name = getattr(ns, 'hsm_name', None)
            if vault_base_url:
                raise CLIError('--vault-name and --id are mutually exclusive.')
            if hsm_name:
                raise CLIError('--hsm-name and --id are mutually exclusive.')

            ident = parse_key_vault_id(identifier)
            setattr(ns, pure_entity_type + '_name', ident.name)
            setattr(ns, 'vault_base_url', ident.vault_url)
            if ident.version and hasattr(ns, pure_entity_type + '_version'):
                setattr(ns, pure_entity_type + '_version', ident.version)
        elif not (name and vault):
            raise CLIError('incorrect usage: --id ID | --vault-name/--hsm-name VAULT/HSM '
                           '--name/-n NAME [--version VERSION]')

    return _validate


def validate_keyvault_resource_id(entity_type):
    def _validate(ns):
        from azure.keyvault.keys._shared import parse_key_vault_id

        pure_entity_type = entity_type.replace('deleted', '')
        name = getattr(ns, pure_entity_type + '_name', None) or getattr(ns, 'name', None)
        vault = getattr(ns, 'vault_base_url', None)
        if not vault:
            vault = getattr(ns, 'hsm_name', None)
        identifier = getattr(ns, 'identifier', None)

        if identifier:
            vault_base_url = getattr(ns, 'vault_base_url', None)
            hsm_name = getattr(ns, 'hsm_name', None)
            if vault_base_url:
                raise CLIError('--vault-name and --id are mutually exclusive.')
            if hsm_name:
                raise CLIError('--hsm-name and --id are mutually exclusive.')

            kv_resource_id = parse_key_vault_id(identifier)
            if getattr(ns, 'command', None) and 'key rotation-policy' in ns.command:
                setattr(ns, 'key_name', kv_resource_id.name)
            elif getattr(ns, 'command', None) and 'certificate' in ns.command:
                setattr(ns, 'certificate_name', kv_resource_id.name)
            else:
                setattr(ns, 'name', kv_resource_id.name)
            setattr(ns, 'vault_base_url', kv_resource_id.vault_url)
            if kv_resource_id.version and (hasattr(ns, pure_entity_type + '_version') or hasattr(ns, 'version')):
                setattr(ns, 'version', kv_resource_id.version)
        elif not (name and vault):
            raise CLIError('incorrect usage: --id ID | --vault-name/--hsm-name VAULT/HSM '
                           '--name/-n NAME [--version VERSION]')

    return _validate


def validate_encryption(ns):
    if ns.data_type == KeyEncryptionDataType.BASE64:
        ns.value = base64.b64decode(ns.value.encode('utf-8'))
    else:
        ns.value = ns.value.encode('utf-8')
    del ns.data_type


def validate_decryption(ns):
    ns.value = base64.b64decode(ns.value)


def validate_key_create(cmd, ns):
    validate_tags(ns)
    set_vault_base_url(ns)
    validate_keyvault_resource_id('key')(ns)
    validate_external_key_id(ns)

    if getattr(ns, 'external_key_id', None):
        # External keys are backed by an External Key Manager (EKM); the service controls the
        # key material, so client-specified key-shape arguments are not supported. Fail fast with
        # a clear error instead of silently ignoring them.
        incompatible = [opt for opt, val in (
            ('--kty', getattr(ns, 'kty', None)),
            ('--size', getattr(ns, 'key_size', None)),
            ('--curve', getattr(ns, 'curve', None)),
            ('--ops', getattr(ns, 'key_ops', None)),
            ('--protection', getattr(ns, 'protection', None)),
            ('--exportable', getattr(ns, 'exportable', None)),
        ) if val is not None]
        if incompatible:
            raise CLIError(
                '{} cannot be used with --external-key-id. External keys are backed by an External '
                'Key Manager and the service controls the key material.'.format(', '.join(incompatible)))
    else:
        validate_key_type(ns)

    process_key_release_policy(cmd, ns)


def validate_external_key_id(ns):
    external_key_id = getattr(ns, 'external_key_id', None)
    if not external_key_id:
        return
    if len(external_key_id) > 64:
        raise CLIError('--external-key-id must be at most 64 characters.')
    if not re.match(r'^[0-9A-Za-z-]+$', external_key_id):
        raise CLIError('--external-key-id may contain only letters, digits, and hyphens.')


def _validate_ekm_path_prefix(path_prefix=None):
    if path_prefix is None:
        return
    if not path_prefix.startswith('/'):
        raise CLIError('--path-prefix must start with "/".')
    if path_prefix.endswith('/'):
        raise CLIError('--path-prefix must not end with "/".')
    if len(path_prefix) > 64:
        raise CLIError('--path-prefix must be at most 64 characters.')
    if not re.match(r'^[A-Za-z0-9/-]+$', path_prefix):
        raise CLIError('--path-prefix may contain only letters, digits, "/" and "-".')


def _normalize_ekm_host(host: str):
    host = (host or '').strip()
    if not host:
        raise CLIError('--host cannot be empty.')
    if '://' in host:
        raise CLIError('--host must not include a URL scheme (use FQDN or FQDN:port).')
    if '/' in host:
        raise CLIError('--host must not include a path (use FQDN or FQDN:port).')

    if ':' not in host:
        return f'{host}:443'

    # Avoid ambiguous parsing for IPv6 literals.
    if host.count(':') != 1:
        raise CLIError('--host must be in the form FQDN or FQDN:port.')

    hostname, port_str = host.split(':', 1)
    if not hostname:
        raise CLIError('--host must be in the form FQDN or FQDN:port.')
    try:
        port = int(port_str)
    except ValueError as ex:
        raise CLIError('--host port must be an integer.') from ex
    if port < 1 or port > 65535:
        raise CLIError('--host port must be between 1 and 65535.')
    return f'{hostname}:{port}'


def _flatten_list(value):
    if value is None:
        return None
    if isinstance(value, list) and value and isinstance(value[0], list):
        flattened = []
        for item in value:
            flattened.extend(item)
        return flattened
    return value


def _load_certificates_as_der_bytes(cert_paths):
    import os
    import ssl

    cert_paths = _flatten_list(cert_paths)
    if not cert_paths:
        return []

    der_certs = []
    for cert_path in cert_paths:
        if not cert_path:
            continue
        expanded = os.path.expanduser(cert_path)
        try:
            with open(expanded, 'rb') as f:
                raw = f.read()
        except OSError as ex:
            raise CLIError("Unable to load certificate file '{}': {}.".format(cert_path, ex.strerror)) from ex

        # PEM may contain multiple cert blocks.
        if b'-----BEGIN CERTIFICATE-----' in raw:
            text = raw.decode('utf-8', errors='ignore')
            begin = '-----BEGIN CERTIFICATE-----'
            end = '-----END CERTIFICATE-----'
            start = 0
            found_any = False
            while True:
                b_idx = text.find(begin, start)
                if b_idx == -1:
                    break
                e_idx = text.find(end, b_idx)
                if e_idx == -1:
                    raise CLIError(f'Invalid PEM certificate in {cert_path}.')
                block = text[b_idx:e_idx + len(end)]
                der_certs.append(ssl.PEM_cert_to_DER_cert(block))
                found_any = True
                start = e_idx + len(end)
            if not found_any:
                raise CLIError(f'Invalid PEM certificate in {cert_path}.')
        else:
            # Assume DER.
            der_certs.append(raw)

    return der_certs


def validate_ekm_connection_base(cmd, ns):  # pylint: disable=unused-argument
    set_vault_base_url(ns)
    if not getattr(ns, 'hsm_name', None) and not getattr(ns, 'identifier', None):
        raise CLIError('Please specify --hsm-name or --id.')


def validate_ekm_connection_create(cmd, ns):
    validate_ekm_connection_base(cmd, ns)
    ns.host = _normalize_ekm_host(ns.host)
    _validate_ekm_path_prefix(getattr(ns, 'path_prefix', None))
    server_ca_certificates = _load_certificates_as_der_bytes(getattr(ns, 'server_ca_certificates', None))
    if not server_ca_certificates:
        raise CLIError('Please specify at least one --server-ca-certificate for EKM connection creation.')
    ns.server_ca_certificates = server_ca_certificates


def validate_ekm_connection_update(cmd, ns):
    validate_ekm_connection_base(cmd, ns)
    if getattr(ns, 'host', None):
        ns.host = _normalize_ekm_host(ns.host)
    _validate_ekm_path_prefix(getattr(ns, 'path_prefix', None))
    if getattr(ns, 'server_ca_certificates', None):
        ns.server_ca_certificates = _load_certificates_as_der_bytes(ns.server_ca_certificates)


# pylint: disable=line-too-long, too-many-locals
def process_certificate_policy(cmd, ns):
    policy = getattr(ns, 'policy', None)
    if policy is None:
        return
    if not isinstance(policy, dict):
        raise CLIError('incorrect usage: policy should be a JSON encoded string '
                       'or can use @{file} to load from a file(e.g.@my_policy.json).')

    secret_properties = policy.get('secret_properties') or {}
    if not secret_properties.get('content_type') \
            and hasattr(ns, 'certificate_bytes') and ns.certificate_bytes:
        from OpenSSL import crypto
        try:
            crypto.load_certificate(crypto.FILETYPE_PEM, ns.certificate_bytes)
            # if we get here, we know it was a PEM file
            secret_properties['content_type'] = 'application/x-pem-file'
        except (ValueError, crypto.Error):
            # else it should be a pfx file
            secret_properties['content_type'] = 'application/x-pkcs12'
        policy['secret_properties'] = secret_properties

    if hasattr(ns, 'validity'):
        x509_certificate_properties = policy.get('x509_certificate_properties') or {}
        if ns.validity:
            x509_certificate_properties['validity_in_months'] = ns.validity
            policy['x509_certificate_properties'] = x509_certificate_properties
        del ns.validity

    policyObj = build_certificate_policy(cmd.cli_ctx, policy)
    ns.policy = policyObj


def build_certificate_policy(cli_ctx, policy: dict):
    from azure.cli.core.profiles import get_sdk
    CertificatePolicy = get_sdk(cli_ctx, ResourceType.DATA_KEYVAULT_CERTIFICATES,
                                'CertificatePolicy', mod='_models')
    CertificateAttributes = get_sdk(cli_ctx, ResourceType.DATA_KEYVAULT_CERTIFICATES,
                                    'CertificateAttributes', mod='_generated_models')
    LifetimeAction = get_sdk(cli_ctx, ResourceType.DATA_KEYVAULT_CERTIFICATES,
                             'LifetimeAction', mod='_models')

    issuer_name = subject = exportable = key_type = key_size = reuse_key = key_curve_name = enhanced_key_usage \
        = key_usage = content_type = validity_in_months = issuer_certificate_type = certificate_transparency = san_emails \
        = san_dns_names = san_user_principal_names = None

    attributes = policy.get('attributes')
    if attributes:
        attributes = CertificateAttributes(
            enabled=attributes.get('enabled'),
            not_before=attributes.get('not_before'),
            expires=attributes.get('expires_on'),
        )

    issuer_parameters = policy.get('issuer_parameters')
    if issuer_parameters:
        issuer_name = issuer_parameters.get('name')
        issuer_certificate_type = issuer_parameters.get('certificate_type')
        certificate_transparency = issuer_parameters.get('certificate_transparency')

    key_properties = policy.get('key_properties')
    if key_properties:
        exportable = key_properties.get('exportable')
        key_type = key_properties.get('key_type')
        key_size = key_properties.get('key_size')
        reuse_key = key_properties.get('reuse_key')
        key_curve_name = key_properties.get('curve')

    lifetime_actions = policy.get('lifetime_actions')
    if lifetime_actions:
        lifetime_actions = [
            LifetimeAction(
                action=item['action'].get('action_type') if item.get('action') else None,
                lifetime_percentage=item['trigger'].get('lifetime_percentage') if item.get('trigger') else None,
                days_before_expiry=item['trigger'].get('days_before_expiry') if item.get('trigger') else None
            )
            for item in lifetime_actions]

    secret_properties = policy.get('secret_properties')
    if secret_properties:
        content_type = secret_properties.get('content_type')

    x509_certificate_properties = policy.get('x509_certificate_properties')
    if x509_certificate_properties:
        subject = x509_certificate_properties.get('subject')
        enhanced_key_usage = x509_certificate_properties.get('ekus')
        subject_alternative_names = x509_certificate_properties.get('subject_alternative_names')
        if subject_alternative_names:
            san_emails = subject_alternative_names.get('emails')
            san_user_principal_names = subject_alternative_names.get('upns')
            san_dns_names = subject_alternative_names.get('dns_names')
        key_usage = x509_certificate_properties.get('key_usage')
        validity_in_months = x509_certificate_properties.get('validity_in_months')

    policyObj = CertificatePolicy(issuer_name=issuer_name, subject=subject, attributes=attributes,
                                  exportable=exportable, key_type=key_type, key_size=key_size, reuse_key=reuse_key,
                                  key_curve_name=key_curve_name, enhanced_key_usage=enhanced_key_usage,
                                  key_usage=key_usage, content_type=content_type, validity_in_months=validity_in_months,
                                  lifetime_actions=lifetime_actions, certificate_type=issuer_certificate_type,
                                  certificate_transparency=certificate_transparency, san_emails=san_emails,
                                  san_dns_names=san_dns_names, san_user_principal_names=san_user_principal_names)
    return policyObj


def process_certificate_import(ns):
    if ns.disabled is not None:
        ns.enabled = not ns.disabled
    del ns.disabled



# --------------------------------------------------------------------------------------------
# Copyright (c) Microsoft Corporation. All rights reserved.
# Licensed under the MIT License. See License.txt in the project root for license information.
# --------------------------------------------------------------------------------------------

import os
import time

from azure.mgmt.core.tools import parse_resource_id

from azure.cli.testsdk.scenario_tests import AllowLargeResponse
from azure.cli.testsdk import (ScenarioTest, ResourceGroupPreparer, JMESPathCheck, LogAnalyticsWorkspacePreparer)
from azure.cli.command_modules.containerapp.tests.latest.common import (write_test_file, clean_up_test_file)

from azure.cli.command_modules.containerapp.tests.latest.common import TEST_LOCATION
from .utils import create_containerapp_env, prepare_containerapp_env_for_app_e2e_tests

TEST_DIR = os.path.abspath(os.path.join(os.path.abspath(__file__), '..'))
# flake8: noqa
# noqa
# pylint: skip-file


class ContainerAppJobsExecutionsTest(ScenarioTest):
    def __init__(self, *arg, **kwargs):
        super().__init__(*arg, random_config_dir=True, **kwargs)

    @AllowLargeResponse(8192)
    @ResourceGroupPreparer(location="northcentralus")
    def test_containerapp_job_executionstest_e2e(self, resource_group):
        import requests
        
        self.cmd('configure --defaults location={}'.format(TEST_LOCATION))
        job = self.create_random_name(prefix='job2', length=24)

        env_id = prepare_containerapp_env_for_app_e2e_tests(self)
        env_rg = parse_resource_id(env_id).get('resource_group')
        env_name = parse_resource_id(env_id).get('name')

        # create a container app environment for a Container App Job resource
        self.cmd('containerapp env show -n {} -g {}'.format(env_name, env_rg), checks=[
            JMESPathCheck('name', env_name)
        ])

        # create a Container App Job resource
        self.cmd("az containerapp job create --resource-group {} --name {} --environment {} --replica-timeout 200 --replica-retry-limit 1 --trigger-type manual --replica-completion-count 1 --parallelism 1 --image mcr.microsoft.com/k8se/quickstart:latest --cpu '0.25' --memory '0.5Gi'".format(resource_group, job, env_id))

        # wait for 60s for the job to be provisioned
        jobProvisioning = True
        timeout = time.time() + 60*1   # 1 minutes from now
        while(jobProvisioning):
            jobProvisioning = self.cmd("az containerapp job show --resource-group {} --name {}".format(resource_group, job)).get_output_in_json()['properties']['provisioningState'] != "Succeeded"
            if(time.time() > timeout):
                break

        # start the job execution
        execution = self.cmd("az containerapp job start --resource-group {} --name {}".format(resource_group, job)).get_output_in_json()
        if "id" in execution:
            # check if the job execution id is in the response
            self.assertEqual(job in execution['id'], True)
        if "name" in execution:
            # check if the job execution name is in the response
            self.assertEqual(job in execution['name'], True)

        # get list of all executions for the job
        executionList = self.cmd("az containerapp job execution list --resource-group {} --name {}".format(resource_group, job)).get_output_in_json()
        self.assertTrue(len(executionList) == 1)
        
        # get single execution for the job
        singleExecution = self.cmd("az containerapp job execution show --resource-group {} --name {} --job-execution-name {}".format(resource_group, job, execution['name'])).get_output_in_json()
        self.assertEqual(job in singleExecution['name'], True)
        
        # start a job execution and stop it
        execution = self.cmd("az containerapp job start --resource-group {} --name {}".format(resource_group, job)).get_output_in_json()
        if "id" in execution:
            # check if the job execution id is in the response
            self.assertEqual(job in execution['id'], True)
        if "name" in execution:
            # check if the job execution name is in the response
            self.assertEqual(job in execution['name'], True)

        # execute twice
        execution2 = self.cmd(
            "az containerapp job start --resource-group {} --name {}".format(resource_group, job)).get_output_in_json()
        executionList = self.cmd(
            "az containerapp job execution list --resource-group {} --name {}".format(resource_group, job), checks=[
                JMESPathCheck('length(@)', 3),
            ]).get_output_in_json()

        for e in executionList:
            self.assertEqual(e['properties']['status'], "Running")

        # stop the most recently started execution
        stopExecution = self.cmd("az containerapp job stop --resource-group {} --name {} --job-execution-name {}".format(resource_group, job, execution['name']))
        # check if the stopExecution response contains the job execution name
        self.assertEqual(execution['name'] in stopExecution.output, True)
        
        # get stopped execution for the job and check status after waiting for 5 seconds to ensure job has stopped
        time.sleep(5)
        singleExecution = self.cmd("az containerapp job execution show --resource-group {} --name {} --job-execution-name {}".format(resource_group, job, execution['name'])).get_output_in_json()
        self.assertEqual(job in singleExecution['name'], True)
        self.assertEqual(singleExecution['properties']['status'], "Stopped")

        executionList = self.cmd(
            "az containerapp job execution list --resource-group {} --name {}".format(resource_group, job), checks=[
                JMESPathCheck('length(@)', 3),
            ]).get_output_in_json()

        # The other one execution is still running
        for e in executionList:
            if e['name'] == singleExecution['name']:
                self.assertEqual(e['properties']['status'], "Stopped")
            else:
                self.assertEqual(e['properties']['status'], "Running")

        # self.cmd("az containerapp job execution show --resource-group {} --name {} --job-execution-name {}".format(resource_group, job, execution2['name']), checks=[
        #     JMESPathCheck('properties.status', 'Running'),
        # ])
        self.cmd("az containerapp job stop --resource-group {} --name {}".format(resource_group, job), expect_failure=False).get_output_in_json()

        # get stopped execution for the job and check status after waiting for 5 seconds to ensure job has stopped
        time.sleep(5)
        executionList = self.cmd("az containerapp job execution list --resource-group {} --name {}".format(resource_group, job), checks=[
            JMESPathCheck('length(@)', 3),
        ]).get_output_in_json()

        for e in executionList:
            self.assertEqual(e['properties']['status'], "Stopped")

    @AllowLargeResponse(8192)
    @ResourceGroupPreparer(location="northcentralus")
    def test_containerapp_stop_job_deprecate_arguments(self, resource_group):
        import requests

        TEST_LOCATION = "northcentralusstage"
        self.cmd('configure --defaults location={}'.format(TEST_LOCATION))

        job = self.create_random_name(prefix='job4', length=24)

        env_id = prepare_containerapp_env_for_app_e2e_tests(self)
        env_rg = parse_resource_id(env_id).get('resource_group')
        env_name = parse_resource_id(env_id).get('name')

        # create a container app environment for a Container App Job resource
        self.cmd('containerapp env show -n {} -g {}'.format(env_name, env_rg), checks=[
            JMESPathCheck('name', env_name)
        ])

        # create a Container App Job resource
        self.cmd("az containerapp job create --resource-group {} --name {} --environment {} --replica-timeout 200 --replica-retry-limit 1 --trigger-type manual --replica-completion-count 1 --parallelism 1 --image mcr.microsoft.com/k8se/quickstart:latest --cpu '0.25' --memory '0.5Gi'".format(resource_group, job, env_id))

        # wait for 60s for the job to be provisioned
        jobProvisioning = True
        timeout = time.time() + 60*1   # 1 minutes from now
        while(jobProvisioning):
            jobProvisioning = self.cmd("az containerapp job show --resource-group {} --name {}".format(resource_group, job)).get_output_in_json()['properties']['provisioningState'] != "Succeeded"
            if(time.time() > timeout):
                break

        # start an execution with custom container information
        customContainerImage = "mcr.microsoft.com/k8se/quickstart:latest"
        customContainerName = "job3-custom-exec"
        execution = self.cmd("az containerapp job start --resource-group {} --name {} --image {} --container-name {} --cpu '0.5' --memory '1Gi'".format(resource_group, job, customContainerImage, customContainerName)).get_output_in_json()
        if "id" in execution:
            # check if the job execution id is in the response
            self.assertEqual(job in execution['id'], True)
        if "name" in execution:
            # check if the job execution name is in the response
            self.assertEqual(job in execution['name'], True)

        # create a string list with job execution name
        execution_name_list = execution['name']

        # stop the execution with deprecated param --execution_name_list
        self.cmd("az containerapp job stop --resource-group {} --name {} --execution-name-list {}".format(resource_group, job, execution['name']), expect_failure=False).get_output_in_json()

    @AllowLargeResponse(8192)
    @ResourceGroupPreparer(location="northcentralus")
    def test_containerapp_job_custom_executionstest_e2e(self, resource_group):
        import requests

        TEST_LOCATION = "northcentralusstage"
        self.cmd('configure --defaults location={}'.format(TEST_LOCATION))

        job = self.create_random_name(prefix='job3', length=24)

        env_id = prepare_containerapp_env_for_app_e2e_tests(self)
        env_rg = parse_resource_id(env_id).get('resource_group')
        env_name = parse_resource_id(env_id).get('name')

        # create a container app environment for a Container App Job resource
        self.cmd('containerapp env show -n {} -g {}'.format(env_name, env_rg), checks=[
            JMESPathCheck('name', env_name)
        ])

        # create a Container App Job resource
        self.cmd("az containerapp job create --resource-group {} --name {} --environment {} --replica-timeout 200 --replica-retry-limit 1 --trigger-type manual --replica-completion-count 1 --parallelism 1 --image mcr.microsoft.com/k8se/quickstart:latest --cpu '0.25' --memory '0.5Gi'".format(resource_group, job, env_id))

        # wait for 60s for the job to be provisioned
        jobProvisioning = True
        timeout = time.time() + 60*1   # 1 minutes from now
        while(jobProvisioning):
            jobProvisioning = self.cmd("az containerapp job show --resource-group {} --name {}".format(resource_group, job)).get_output_in_json()['properties']['provisioningState'] != "Succeeded"
            if(time.time() > timeout):
                break

        # start an execution with custom container information
        customContainerImage = "mcr.microsoft.com/k8se/quickstart:latest"
        customContainerName = "job3-custom-exec"
        execution = self.cmd("az containerapp job start --resource-group {} --name {} --image {} --container-name {} --cpu '0.5' --memory '1Gi'".format(resource_group, job, customContainerImage, customContainerName)).get_output_in_json()
        if "id" in execution:
            # check if the job execution id is in the response
            self.assertEqual(job in execution['id'], True)
        if "name" in execution:
            # check if the job execution name is in the response
            self.assertEqual(job in execution['name'], True)

        # get the execution and check if the custom container information is present
        self.cmd("az containerapp job execution show --resource-group {} --name {} --job-execution-name {}".format(resource_group, job, execution['name']), checks=[
            JMESPathCheck('properties.template.containers[0].name', customContainerName),
            JMESPathCheck('properties.template.containers[0].image', customContainerImage),
            JMESPathCheck('properties.template.containers[0].resources.cpu', '0.5'),
            JMESPathCheck('properties.template.containers[0].resources.memory', '1Gi')
        ])

        # yaml file to start a job execution
        containerappjobexecution_yaml_text = f"""
            containers:
                - env:
                    - name: MY_ENV_VAR
                      value: hello
                  image: mcr.microsoft.com/k8se/quickstart-jobs:latest
                  name: test-yaml-execution
                  resources:
                    cpu: 0.5
                    memory: 1Gi
            initContainers:
                - command:
                    - /bin/sh
                    - -c
                    - sleep 10
                  image: k8seteste2e.azurecr.io/e2e-apps/kuar:green
                  name: simple-sleep-container
                  resources:
                    cpu: "0.25"
                    memory: 0.5Gi
            """
        containerappjob_file_name = f"{self._testMethodName}_containerappjob.yml"
        write_test_file(containerappjob_file_name, containerappjobexecution_yaml_text)

        # start job execution with yaml file
        execution = self.cmd("az containerapp job start --resource-group {} --name {} --yaml {}".format(resource_group, job, containerappjob_file_name)).get_output_in_json()
        self.assertEqual(job in execution['id'], True)
        self.assertEqual(job in execution['name'], True)            

        # get the execution and check if the custom container information is present
        self.cmd("az containerapp job execution show --resource-group {} --name {} --job-execution-name {}".format(resource_group, job, execution['name']), checks=[
            JMESPathCheck('properties.template.containers[0].name', "test-yaml-execution"),
            JMESPathCheck('properties.template.containers[0].image', "mcr.microsoft.com/k8se/quickstart-jobs:latest"),
            JMESPathCheck('properties.template.containers[0].resources.cpu', '0.5'),
            JMESPathCheck('properties.template.containers[0].resources.memory', '1Gi'),
            JMESPathCheck('properties.template.initContainers[0].name', "simple-sleep-container"),
            JMESPathCheck('properties.template.initContainers[0].image', "k8seteste2e.azurecr.io/e2e-apps/kuar:green"),
            JMESPathCheck('properties.template.initContainers[0].resources.cpu', '0.25'),
            JMESPathCheck('properties.template.initContainers[0].resources.memory', '0.5Gi')
        ])
        clean_up_test_file(containerappjob_file_name)

    @AllowLargeResponse(8192)
    @ResourceGroupPreparer(location="northcentralus")
    def test_containerappjob_start_with_command_and_args_e2e(self, resource_group):
        """
        Test for GitHub issue https://github.com/microsoft/azure-container-apps/issues/1360
        This test validates that --command and --args are properly applied when using
        'az containerapp job start' without specifying --image (using the job's default image).
        """

        TEST_LOCATION = "northcentralusstage"
        self.cmd('configure --defaults location={}'.format(TEST_LOCATION))

        job = self.create_random_name(prefix='job-cmd', length=24)

        env_id = prepare_containerapp_env_for_app_e2e_tests(self)
        env_rg = parse_resource_id(env_id).get('resource_group')
        env_name = parse_resource_id(env_id).get('name')

        # create a container app environment for a Container App Job resource
        self.cmd('containerapp env show -n {} -g {}'.format(env_name, env_rg), checks=[
            JMESPathCheck('name', env_name)
        ])

        # Create a Container App Job resource with a default command
        # Using an image that supports running custom commands (alpine with sh)
        self.cmd("az containerapp job create --resource-group {} --name {} --environment {} --replica-timeout 200 --replica-retry-limit 1 --trigger-type manual --replica-completion-count 1 --parallelism 1 --image mcr.microsoft.com/k8se/quickstart-jobs:latest --cpu '0.25' --memory '0.5Gi'".format(resource_group, job, env_id))

        # wait for 60s for the job to be provisioned
        jobProvisioning = True
        timeout = time.time() + 60*1   # 1 minutes from now
        while(jobProvisioning):
            jobProvisioning = self.cmd("az containerapp job show --resource-group {} --name {}".format(resource_group, job)).get_output_in_json()['properties']['provisioningState'] != "Succeeded"
            if(time.time() > timeout):
                break

        # Test 1: Start job execution with --command and --args WITHOUT specifying --image
        customCommand = ["echo"]
        customArgs = ["hello-from-custom-command"]
        execution = self.cmd("az containerapp job start --resource-group {} --name {} --command {} --args {}".format(
            resource_group, job, " ".join(customCommand), " ".join(customArgs))).get_output_in_json()

        if "name" in execution:
            # Verify the execution was created
            self.assertEqual(job in execution['name'], True)

        # Get the execution and check if the custom command and args are present
        # Also validate the image matches the one from 'az containerapp job create'
        self.cmd("az containerapp job execution show --resource-group {} --name {} --job-execution-name {}".format(resource_group, job, execution['name']), checks=[
            JMESPathCheck('properties.template.containers[0].image', 'mcr.microsoft.com/k8se/quickstart-jobs:latest'),
            JMESPathCheck('properties.template.containers[0].command[0]', customCommand[0]),
            JMESPathCheck('properties.template.containers[0].args[0]', customArgs[0]),
        ])

        # Test 2: For comparison, verify that --command and --args work when --image IS specified
        customContainerImage = "mcr.microsoft.com/k8se/quickstart-jobs:latest"
        execution_with_image = self.cmd("az containerapp job start --resource-group {} --name {} --image {} --command {} --args {}".format(
            resource_group, job, customContainerImage, " ".join(customCommand), " ".join(customArgs))).get_output_in_json()

        if "name" in execution_with_image:
            self.assertEqual(job in execution_with_image['name'], True)

        self.cmd("az containerapp job execution show --resource-group {} --name {} --job-execution-name {}".format(resource_group, job, execution_with_image['name']), checks=[
            JMESPathCheck('properties.template.containers[0].image', customContainerImage),
            JMESPathCheck('properties.template.containers[0].command[0]', customCommand[0]),
            JMESPathCheck('properties.template.containers[0].args[0]', customArgs[0]),
        ])

    @AllowLargeResponse(8192)
    @ResourceGroupPreparer(location="northcentralus")
    @LogAnalyticsWorkspacePreparer(location="eastus", get_shared_key=True)
    def test_containerappjob_create_with_yaml(self, resource_group, laworkspace_customer_id, laworkspace_shared_key):
        self.cmd('configure --defaults location={}'.format(TEST_LOCATION))

        env = self.create_random_name(prefix='env', length=24)
        job = self.create_random_name(prefix='yaml', length=24)
        storage = self.create_random_name(prefix='storage', length=24)
        share = self.create_random_name(prefix='share', length=24)

        self.cmd(
            f'az storage account create --resource-group {resource_group}  --name {storage} --location {TEST_LOCATION} --kind StorageV2 --sku Standard_LRS --enable-large-file-share --output none')
        self.cmd(
            f'az storage share-rm create --resource-group {resource_group}  --storage-account {storage} --name {share} --quota 1024 --enabled-protocols SMB --output none')

        create_containerapp_env(self, env, resource_group, logs_workspace=laworkspace_customer_id, logs_workspace_shared_key=laworkspace_shared_key)
        containerapp_env = self.cmd(
            'containerapp env show -g {} -n {}'.format(resource_group, env)).get_output_in_json()

        account_key = self.cmd(f'az storage account keys list -g {resource_group} -n {storage} --query "[0].value" '
                               '-otsv').output.strip()
        self.cmd(
            f'az containerapp env storage set -g {resource_group} -n {env} -a {storage} -k {account_key} -f {share} --storage-name {share} --access-mode ReadWrite')

        user_identity_name = self.create_random_name(prefix='containerapp-user', length=24)
        user_identity = self.cmd(
            'identity create -g {} -n {}'.format(resource_group, user_identity_name)).get_output_in_json()
        user_identity_id = user_identity['id']

        # test job create with yaml
        containerappjob_yaml_text = f"""
                location: {TEST_LOCATION}
                properties:
                    environmentId: {containerapp_env["id"]}
                    configuration:
                        dapr: null
                        eventTriggerConfig: null
                        manualTriggerConfig:
                            parallelism: 1
                            replicaCompletionCount: 1
                        registries: null
                        replicaRetryLimit: 1
                        replicaTimeout: 100
                        scheduleTriggerConfig: null
                        secrets: null
                        triggerType: Manual
                    template:
                        containers:
                        - env:
                            - name: MY_ENV_VAR
                              value: hello
                          image: mcr.microsoft.com/k8se/quickstart-jobs:latest
                          name: anfranci-azclitest-acaj1
                          resources:
                            cpu: 0.5
                            ephemeralStorage: 1Gi
                            memory: 1Gi
                          volumeMounts:
                            - mountPath: /mnt/data
                              volumeName: azure-files-volume
                              subPath: sub
                        initContainers:
                        - command:
                            - /bin/sh
                            - -c
                            - sleep 150
                          image: k8seteste2e.azurecr.io/e2e-apps/kuar:green
                          name: simple-sleep-container
                          probes:
                          - type: liveness
                            httpGet:
                                path: "/health"
                                port: 8080
                                httpHeaders:
                                    - name: "Custom-Header"
                                      value: "liveness probe"
                            initialDelaySeconds: 7
                            periodSeconds: 3
                            resources:
                                cpu: "0.25"
                                memory: 0.5Gi
                        volumes:
                        - name: azure-files-volume
                          storageType: AzureFile
                          storageName: {share}
                          mountOptions: uid=999,gid=999
                    workloadProfileName: null
                identity:
                  type: UserAssigned
                  userAssignedIdentities:
                    {user_identity_id}: {{}}
                """
        containerappjob_file_name = f"{self._testMethodName}_containerappjob.yml"

        write_test_file(containerappjob_file_name, containerappjob_yaml_text)
        self.cmd(f'containerapp job create -n {job} -g {resource_group} --environment {env} --yaml {containerappjob_file_name}', checks=[
            JMESPathCheck("properties.provisioningState", "Succeeded"),
            JMESPathCheck("properties.configuration.triggerType", "Manual", case_sensitive=False),
            JMESPathCheck('properties.configuration.replicaTimeout', 100),
            JMESPathCheck('properties.configuration.replicaRetryLimit', 1),
            JMESPathCheck('properties.template.containers[0].image', "mcr.microsoft.com/k8se/quickstart-jobs:latest"),
            JMESPathCheck('properties.template.containers[0].resources.cpu', "0.5"),
            JMESPathCheck('properties.template.containers[0].resources.memory', "1Gi"),
            JMESPathCheck('identity.type', "UserAssigned"),
            JMESPathCheck('properties.template.volumes[0].storageType', 'AzureFile'),
            JMESPathCheck('properties.template.volumes[0].storageName', share),
            JMESPathCheck('properties.template.volumes[0].name', 'azure-files-volume'),
            JMESPathCheck('properties.template.volumes[0].mountOptions', 'uid=999,gid=999'),
            JMESPathCheck('properties.template.containers[0].volumeMounts[0].subPath', 'sub'),
            JMESPathCheck('properties.template.containers[0].volumeMounts[0].mountPath', '/mnt/data'),
            JMESPathCheck('properties.template.containers[0].volumeMounts[0].volumeName', 'azure-files-volume')
        ])

        self.cmd(f'containerapp job show -g {resource_group} -n {job}', checks=[
            JMESPathCheck("properties.provisioningState", "Succeeded"),
            JMESPathCheck("properties.configuration.triggerType", "Manual", case_sensitive=False),
            JMESPathCheck('properties.configuration.replicaTimeout', 100),
            JMESPathCheck('properties.configuration.replicaRetryLimit', 1),
            JMESPathCheck('properties.template.containers[0].image', "mcr.microsoft.com/k8se/quickstart-jobs:latest"),
            JMESPathCheck('properties.template.containers[0].resources.cpu', "0.5"),
            JMESPathCheck('properties.template.containers[0].resources.memory', "1Gi"),
            JMESPathCheck('identity.type', "UserAssigned"),
            JMESPathCheck('properties.template.volumes[0].storageType', 'AzureFile'),
            JMESPathCheck('properties.template.volumes[0].storageName', share),
            JMESPathCheck('properties.template.volumes[0].name', 'azure-files-volume'),
            JMESPathCheck('properties.template.volumes[0].mountOptions', 'uid=999,gid=999'),
            JMESPathCheck('properties.template.containers[0].volumeMounts[0].subPath', 'sub'),
            JMESPathCheck('properties.template.containers[0].volumeMounts[0].mountPath', '/mnt/data'),
            JMESPathCheck('properties.template.containers[0].volumeMounts[0].volumeName', 'azure-files-volume')
        ])

        # wait for provisioning state of job to be succeeded before updating
        jobProvisioning = True
        timeout = time.time() + 60 * 1  # 1 minutes from now
        while (jobProvisioning):
            jobProvisioning = self.cmd("az containerapp job show --resource-group {} --name {}".format(resource_group,
                                                                                                       job)).get_output_in_json()[
                                  'properties']['provisioningState'] != "Succeeded"
            if (time.time() > timeout):
                break

        # test container app job update with yaml
        containerappjob_yaml_text = f"""
                location: {TEST_LOCATION}
                properties:
                    environmentId: {containerapp_env["id"]}
                    configuration:
                        dapr: null
                        eventTriggerConfig: null
                        manualTriggerConfig:
                            parallelism: 1
                            replicaCompletionCount: 1
                        registries: null
                        replicaRetryLimit: 1
                        replicaTimeout: 200
                        scheduleTriggerConfig: null
                        secrets: null
                        triggerType: Manual
                    template:
                        containers:
                        - env:
                            - name: MY_ENV_VAR
                              value: hello
                          image: mcr.microsoft.com/k8se/quickstart-jobs:latest
                          name: anfranci-azclitest-acaj1
                          resources:
                            cpu: 0.75
                            ephemeralStorage: 1Gi
                            memory: 1.5Gi
                          volumeMounts:
                            - mountPath: /mnt/data
                              volumeName: azure-files-volume
                              subPath: sub2
                        initContainers:
                        - command:
                            - /bin/sh
                            - -c
                            - sleep 150
                          image: k8seteste2e.azurecr.io/e2e-apps/kuar:green
                          name: simple-sleep-container
                        probes:
                        - type: liveness
                          httpGet:
                            path: "/health"
                            port: 8080
                            httpHeaders:
                                - name: "Custom-Header"
                                  value: "liveness probe"
                            initialDelaySeconds: 7
                            periodSeconds: 3
                            resources:
                                cpu: "0.25"
                                memory: 0.5Gi
                        volumes:
                        - name: azure-files-volume
                          storageType: AzureFile
                          storageName: {share}
                          mountOptions: uid=1000,gid=1000
                """
        write_test_file(containerappjob_file_name, containerappjob_yaml_text)

        self.cmd(f'containerapp job update -n {job} -g {resource_group} --yaml {containerappjob_file_name}', checks=[
            JMESPathCheck('properties.provisioningState', "Succeeded"),
            JMESPathCheck("properties.configuration.triggerType", "Manual", case_sensitive=False),
            JMESPathCheck('properties.configuration.replicaTimeout', 200),
            JMESPathCheck('properties.configuration.replicaRetryLimit', 1),
            JMESPathCheck('properties.template.containers[0].image', "mcr.microsoft.com/k8se/quickstart-jobs:latest"),
            JMESPathCheck('properties.template.containers[0].resources.cpu', "0.75"),
            JMESPathCheck('properties.template.containers[0].resources.memory', "1.5Gi"),
            JMESPathCheck('properties.template.volumes[0].storageType', 'AzureFile'),
            JMESPathCheck('properties.template.volumes[0].storageName', share),
            JMESPathCheck('properties.template.volumes[0].name', 'azure-files-volume'),
            JMESPathCheck('properties.template.volumes[0].mountOptions', 'uid=1000,gid=1000'),
            JMESPathCheck('properties.template.containers[0].volumeMounts[0].subPath', 'sub2'),
            JMESPathCheck('properties.template.containers[0].volumeMounts[0].mountPath', '/mnt/data'),
            JMESPathCheck('properties.template.containers[0].volumeMounts[0].volumeName', 'azure-files-volume'),
        ])

        self.cmd(f'containerapp job show -g {resource_group} -n {job}', checks=[
            JMESPathCheck("properties.configuration.triggerType", "Manual", case_sensitive=False),
            JMESPathCheck('properties.configuration.replicaTimeout', 200),
            JMESPathCheck('properties.configuration.replicaRetryLimit', 1),
            JMESPathCheck('properties.template.containers[0].image', "mcr.microsoft.com/k8se/quickstart-jobs:latest"),
            JMESPathCheck('properties.template.containers[0].resources.cpu', "0.75"),
            JMESPathCheck('properties.template.containers[0].resources.memory', "1.5Gi"),
            JMESPathCheck('properties.template.volumes[0].storageType', 'AzureFile'),
            JMESPathCheck('properties.template.volumes[0].storageName', share),
            JMESPathCheck('properties.template.volumes[0].name', 'azure-files-volume'),
            JMESPathCheck('properties.template.volumes[0].mountOptions', 'uid=1000,gid=1000'),
            JMESPathCheck('properties.template.containers[0].volumeMounts[0].subPath', 'sub2'),
            JMESPathCheck('properties.template.containers[0].volumeMounts[0].mountPath', '/mnt/data'),
            JMESPathCheck('properties.template.containers[0].volumeMounts[0].volumeName', 'azure-files-volume'),
        ])

        # wait for provisioning state of job to be succeeded before updating
        jobProvisioning = True
        timeout = time.time() + 60 * 1  # 1 minutes from now
        while (jobProvisioning):
            jobProvisioning = self.cmd("az containerapp job show --resource-group {} --name {}".format(resource_group,
                                                                                                       job)).get_output_in_json()[
                                  'properties']['provisioningState'] != "Succeeded"
            if (time.time() > timeout):
                break

        # test update for job with yaml not containing environmentId
        containerappjob_yaml_text = f"""
                                properties:
                                  configuration:
                                    replicaTimeout: 300
                                """
        write_test_file(containerappjob_file_name, containerappjob_yaml_text)

        self.cmd(f'containerapp job update -n {job} -g {resource_group} --yaml {containerappjob_file_name} --no-wait')

        self.cmd(f'containerapp job show -g {resource_group} -n {job}', checks=[
            JMESPathCheck("properties.environmentId", containerapp_env["id"]),
            JMESPathCheck("properties.configuration.replicaTimeout", 300)
        ])
        clean_up_test_file(containerappjob_file_name)

    @AllowLargeResponse(8192)
    @ResourceGroupPreparer(location="northcentralus")
    @LogAnalyticsWorkspacePreparer(location="eastus", get_shared_key=True)
    def test_containerappjob_eventtriggered_create_with_yaml(self, resource_group, laworkspace_customer_id, laworkspace_shared_key):
        self.cmd('configure --defaults location={}'.format(TEST_LOCATION))

        env = self.create_random_name(prefix='env', length=24)
        job = self.create_random_name(prefix='yaml', length=24)

        create_containerapp_env(self, env, resource_group, logs_workspace=laworkspace_customer_id, logs_workspace_shared_key=laworkspace_shared_key)
        containerapp_env = self.cmd(
            'containerapp env show -g {} -n {}'.format(resource_group, env)).get_output_in_json()

        user_identity_name = self.create_random_name(prefix='containerapp-user', length=24)
        user_identity = self.cmd(
            'identity create -g {} -n {}'.format(resource_group, user_identity_name)).get_output_in_json()
        user_identity_id = user_identity['id']

        # test job create with yaml
        containerappjob_yaml_text = f"""
                location: {TEST_LOCATION}
                properties:
                    environmentId: {containerapp_env["id"]}
                    configuration:
                        eventTriggerConfig:
                            replicaCompletionCount: 1
                            parallelism: 1
                            scale:
                                minExecutions: 1
                                maxExecutions: 11
                                rules:
                                - name: github-runner-test
                                  type: github-runner
                                  metadata:
                                    github-runner: https://api.github.com
                                    owner: test_org
                                    runnerScope: repo
                                    repos: test_repo
                                    targetWorkflowQueueLength: "1"
                                  auth:
                                    - secretRef: personal-access-token
                                      triggerParameter: personalAccessToken
                        replicaRetryLimit: 1
                        replicaTimeout: 100
                        secrets:
                            - name: personal-access-token
                              value: test_personal_access_token
                        triggerType: Event
                    template:
                        containers:
                        - env:
                            - name: ACCESS_TOKEN
                              secretRef: personal-access-token
                            - name: DISABLE_RUNNER_UPDATE
                              value: "true"
                            - name: RUNNER_SCOPE
                              value: repo
                            - name: ORG_NAME
                              value: test_org
                            - name: ORG_RUNNER
                              value: "false"
                            - name: RUNNER_WORKDIR
                              value: /tmp/runner
                            - name: REPO_URL
                              value: https://github.com/test_org/test_repo
                          image: mcr.microsoft.com/k8se/quickstart-jobs:latest
                          name: eventdriventjob
                          resources:
                            cpu: 0.5
                            ephemeralStorage: 1Gi
                            memory: 1Gi
                          volumeMounts:
                            - volumeName: workdir
                              mountPath: /tmp/github-runner-your-repo
                            - volumeName: dockersock
                              mountPath: /var/run/docker.sock
                        volumes:
                            - name: workdir
                              storageType: EmptyDir
                            - name: dockersock
                              storageType: EmptyDir
                identity:
                  type: UserAssigned
                  userAssignedIdentities:
                    {user_identity_id}: {{}}
                """
        containerappjob_file_name = f"{self._testMethodName}_containerappjob.yml"

        write_test_file(containerappjob_file_name, containerappjob_yaml_text)
        self.cmd(
            f'containerapp job create -n {job} -g {resource_group} --environment {env} --yaml {containerappjob_file_name}')

        self.cmd(f'containerapp job show -g {resource_group} -n {job}', checks=[
            JMESPathCheck("properties.provisioningState", "Succeeded"),
            JMESPathCheck("properties.configuration.triggerType", "Event", case_sensitive=False),
            JMESPathCheck('properties.configuration.replicaTimeout', 100),
            JMESPathCheck('properties.configuration.replicaRetryLimit', 1),
            JMESPathCheck('properties.template.containers[0].image', "mcr.microsoft.com/k8se/quickstart-jobs:latest"),
            JMESPathCheck('properties.template.containers[0].resources.cpu', "0.5"),
            JMESPathCheck('properties.template.containers[0].resources.memory', "1Gi"),
            JMESPathCheck('properties.configuration.eventTriggerConfig.replicaCompletionCount', 1),
            JMESPathCheck('properties.configuration.eventTriggerConfig.parallelism', 1),
            JMESPathCheck('properties.configuration.eventTriggerConfig.scale.minExecutions', 1),
            JMESPathCheck('properties.configuration.eventTriggerConfig.scale.maxExecutions', 11),
            JMESPathCheck('properties.configuration.eventTriggerConfig.scale.rules[0].type', "github-runner"),
            JMESPathCheck('properties.configuration.eventTriggerConfig.scale.rules[0].metadata.runnerScope', "repo"),
            JMESPathCheck('properties.configuration.eventTriggerConfig.scale.rules[0].auth[0].secretRef',
                          "personal-access-token"),
            JMESPathCheck('identity.type', "UserAssigned"),
        ])

        # wait for provisioning state of job to be succeeded before updating
        jobProvisioning = True
        timeout = time.time() + 60 * 1  # 1 minutes from now
        while (jobProvisioning):
            jobProvisioning = self.cmd("az containerapp job show --resource-group {} --name {}".format(resource_group,
                                                                                                       job)).get_output_in_json()[
                                  'properties']['provisioningState'] != "Succeeded"
            if (time.time() > timeout):
                break

        # test container app job update with yaml
        containerappjob_yaml_text = f"""
                location: {TEST_LOCATION}
                properties:
                    environmentId: {containerapp_env["id"]}
                    configuration:
                        eventTriggerConfig:
                            replicaCompletionCount: 2
                            parallelism: 2
                            scale:
                                minExecutions: 0
                                maxExecutions: 95
                                rules:
                                - name: github-runner-testv2
                                  type: github-runner
                                  metadata:
                                    github-runner: https://api.github.com
                                    owner: test_org_1
                                    runnerScope: repo
                                    repos: test_repo_1
                                    targetWorkflowQueueLength: "1"
                                  auth:
                                    - secretRef: personal-access-token
                                      triggerParameter: personalAccessToken
                        replicaRetryLimit: 2
                        replicaTimeout: 200
                        secrets:
                            - name: personal-access-token
                              value: test_personal_access_token
                        triggerType: Event
                """
        write_test_file(containerappjob_file_name, containerappjob_yaml_text)

        self.cmd(f'containerapp job update -n {job} -g {resource_group} --yaml {containerappjob_file_name} --no-wait')

        self.cmd(f'containerapp job show -g {resource_group} -n {job}', checks=[
            JMESPathCheck("properties.configuration.triggerType", "Event", case_sensitive=False),
            JMESPathCheck('properties.configuration.replicaTimeout', 200),
            JMESPathCheck('properties.configuration.replicaRetryLimit', 2),
            JMESPathCheck('properties.template.containers[0].image', "mcr.microsoft.com/k8se/quickstart-jobs:latest"),
            JMESPathCheck('properties.configuration.eventTriggerConfig.replicaCompletionCount', 2),
            JMESPathCheck('properties.configuration.eventTriggerConfig.parallelism', 2),
            JMESPathCheck('properties.configuration.eventTriggerConfig.scale.minExecutions', 0),
            JMESPathCheck('properties.configuration.eventTriggerConfig.scale.maxExecutions', 95),
            JMESPathCheck('properties.configuration.eventTriggerConfig.scale.rules[0].name', "github-runner-testv2"),
            JMESPathCheck('properties.configuration.eventTriggerConfig.scale.rules[0].metadata.runnerScope', "repo"),
            JMESPathCheck('properties.configuration.eventTriggerConfig.scale.rules[0].metadata.owner', "test_org_1"),
            JMESPathCheck('properties.configuration.eventTriggerConfig.scale.rules[0].auth[0].secretRef',
                          "personal-access-token"),
        ])
        clean_up_test_file(containerappjob_file_name)